# [Epoch handling and timescales conversions](@id tutorial_01_epochs)

`Basic` gives the capability to represent time in different manners and to convert their representation between different timescales.

In this tutorial, the general workflow to be followed when dealing with time representations and their transformations is presented.

The first step is to load the `Tempo` module:


```julia
using Basic
using Basic.Tempo
```

This package contains the basic routines needed to handle `Epochs` and their transformations
as well as some additional types to represent `Date`s, `Time` and `DateTime` objects.

In particular, the following types, methods and constants are available from `Tempo`:


```julia
println(names(Basic.Tempo; all=false, imported=true))
```

    [Symbol("@timescale"), :BarycentricCoordinateTime, :BarycentricDynamicalTime, :CoordinatedUniversalTime, :DJ2000, :DJM0, :DMJD, :Date, :DateTime, :Epoch, :EpochConversionError, :FunctionWrapper, :GPS, :GeocentricCoordinateTime, :GlobalPositioningSystemTime, :HighPrecisionBarycentricDynamicalTime, :InternationalAtomicTime, :J2000, :JD, :JulianDate, :JulianDate2000, :MJ2000, :MJD, :ModifiedJulianDate, :ModifiedJulianDate2000, :TAI, :TCB, :TCG, :TDB, :TDBH, :TIMESCALES, :TT, :Tempo, :TerrestrialTime, :Time, :TimeSystem, :UT1, :UTC, :UniversalTime, :add_timescale, :add_vertex!, :cal2jd, :day, :find_dayinyear, :fraction_of_day, :fraction_of_second, :get_node_id, :hour, :isleapyear, :j2000, :j2000c, :j2000s, :minute, :month, :parse_iso, :second, :second_in_day, :timescale, :value, :year]


**Then, how and epoch may be represented when using `Basic`?** Well, depends on the actual use case.

As already said, there are different time representations in `Basic` and they do
depends on the actual information needed for the particular application. In general, when
dealing with _space_ applications, the a time representation shall _embed_ different informations:

1. The representation type (e.g. Gregorian or Julian calendar representation);
2. The origin (e.g. J2000, JD, MJD, ...);
3. The time scale (e.g. TAI, TT, TDB, UTC, UT, ...).

These three informations are considered when building an `Epoch`, which is the most complete
time representation available within `Basic` and is the one suggested to the user. Then, to 
handle such _complex_ time representation, different constructors are available for `Epoch`
and a _time transformation system_ is available. However, there are some assumptions that
are considered:

1. The (internal) representation type of `Epoch` is always exploiting Julian calendar;
2. The (internal) `Epoch` origin is always `J2000`, i.e. 01 Jan 2000 at noon;
3. The `Epoch` timescale could be any of the _time transformation system_ ones.
4. The default timescale is `TDB`.

## Epoch creation

To create a new `Epoch` object, there are different ways:

### From a ISO string (without scale):


```julia
e = Epoch("2022-01-02T06:30:00.0")
```


    2022-01-02T06:29:59.000 TDB


### From ISO string (with scale):


```julia
e = Epoch("2022-01-02T06:30:00.0 TT")
```


    2022-01-02T06:29:59.000 TT


### From a (partial) ISO string:


```julia
e = Epoch("2020-01-01")
```


    2020-01-01T00:00:00.000 TDB



```julia
e = Epoch("2021-01-30T01")
```


    2021-01-30T01:00:00.000 TDB



```julia
e = Epoch("2022-06-12 UTC")
```


    2022-06-12T00:00:00.000 UTC


### From a Julian date:


```julia
e = Epoch("JD 2451545.0")
```


    2000-01-01T12:00:00.000 TDB


### From a Modified Julian date:


```julia
e = Epoch("MJD 51544.5")
```


    2000-01-01T12:00:00.000 TDB


### From Julian days since J2000:


```julia
e = Epoch("9.0 TT")
```


    2000-01-10T12:00:00.000 TT


### From seconds since J2000:


```julia
e = Epoch(60.0, TAI)
```


    2000-01-01T12:01:00.000 TAI


### From `DateTime`:


```julia
dt = DateTime(2001, 6, 15, 0, 0, 0, 0.0)
e = Epoch(dt, TT)
```


    2001-06-15T00:00:00.000 TT


## Epoch transformations

The epoch transformations are allowed by means of a **directed** graph of timescales available in
`Tempo`. While the `offset_xxx` functions provides the offsets in seconds between two timescales 
(e.g. `offset_tai2tt` provide the offset in seconds to convert from `TAI` to `TT`), there
is a simple, effective and efficient entrypoint to all time transformations: the `convert` method.


```julia
e = Epoch(90.0, TT)
```


    2000-01-01T12:01:29.000 TT



```julia
# Convert to TAI
eTAI = convert(TAI, e)
```


    2000-01-01T12:00:57.816 TAI



```julia
# To high-precision TDB
eTDBH = convert(TDBH, eTAI)
```


    2000-01-01T12:01:29.999 TDBH


Note that e time transformation system is a **directed** graph! To allow to transform back and forth
a given timescale, two transformations are necessary. An error will be displayed in case a 
single transformation is assigned and the reverse one is called. 

## Timescales transformation example

The objective of this example is to show how to use the `Tempo` module to compute the offset
between a given timescale and all the others registered in the time transformation system.


```julia
using Basic.Tempo, Plots 
using Dates: DateTime as DatesDT

# This is an utility function needed for plotting purposes
# Transforms Tempo DateTimes in Dates DateTimes, which are already supported by Plots 
function DatesDT(dt::Tempo.DateTime)
    return DatesDT(
        dt.date.year, dt.date.month, dt.date.day, 
        dt.time.hour, dt.time.minute, dt.time.second, 0.0)
end
nothing
```


```julia
const REF = TAI  # Declare the reference timescale
```


    TAI


First, we create an epoch grid with a discretization of 30 days:


```julia
eREF = collect(Epoch("1974-01-01 $REF") : 30*86400.0 : Epoch("2024-01-01 $REF"));
```

Then, for plotting purposes, we transform the epochs into `Tempo.DateTime` types and then
use the utility function to transform to `Dates.DateTime`:


```julia
x = [DatesDT(Tempo.DateTime(ei)) for ei in eREF];
```

Now, we create a new empty figure, where to plot all the offsets:


```julia
p = plot();
```

Finally, we plot all the offsets and adjust the axes:


```julia
# plot the offset w.r.t. ref
for scale in (TAI, TT, TDB, TCB, TCG, Tempo.UTC, UT1, GPS)
    dex = [convert(scale, ei).seconds for ei in eREF] - [ei.seconds for ei in eREF]
    plot!(x[2:end], dex[2:end], label="$scale", width=2);
end
ylabel!("Difference in Time to $REF (seconds)")
plot!(
    size=(800, 600), xtickfontsize=10, ytickfontsize=10, guidefontsize=12,
    left_margin=0.5*Plots.cm, bottom_margin=0.5*Plots.cm, xrotation=60
)
```


<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" width="800" height="600" viewBox="0 0 3200 2400">
<defs>
  <clipPath id="clip410">
    <rect x="0" y="0" width="3200" height="2400"/>
  </clipPath>
</defs>
<path clip-path="url(#clip410)" d="
M0 2400 L3200 2400 L3200 0 L0 0  Z
  " fill="#ffffff" fill-rule="evenodd" fill-opacity="1"/>
<defs>
  <clipPath id="clip411">
    <rect x="640" y="0" width="2241" height="2241"/>
  </clipPath>
</defs>
<path clip-path="url(#clip410)" d="
M346.895 2008.4 L3152.76 2008.4 L3152.76 47.2441 L346.895 47.2441  Z
  " fill="#ffffff" fill-rule="evenodd" fill-opacity="1"/>
<defs>
  <clipPath id="clip412">
    <rect x="346" y="47" width="2807" height="1962"/>
  </clipPath>
</defs>
<polyline clip-path="url(#clip412)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="
  368.889,2008.4 368.889,47.2441 
  "/>
<polyline clip-path="url(#clip412)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="
  1059.07,2008.4 1059.07,47.2441 
  "/>
<polyline clip-path="url(#clip412)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="
  1749.24,2008.4 1749.24,47.2441 
  "/>
<polyline clip-path="url(#clip412)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="
  2439.42,2008.4 2439.42,47.2441 
  "/>
<polyline clip-path="url(#clip412)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="
  3129.75,2008.4 3129.75,47.2441 
  "/>
<polyline clip-path="url(#clip410)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="
  346.895,2008.4 3152.76,2008.4 
  "/>
<polyline clip-path="url(#clip410)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="
  368.889,2008.4 368.889,1989.5 
  "/>
<polyline clip-path="url(#clip410)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="
  1059.07,2008.4 1059.07,1989.5 
  "/>
<polyline clip-path="url(#clip410)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="
  1749.24,2008.4 1749.24,1989.5 
  "/>
<polyline clip-path="url(#clip410)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="
  2439.42,2008.4 2439.42,1989.5 
  "/>
<polyline clip-path="url(#clip410)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="
  3129.75,2008.4 3129.75,1989.5 
  "/>
<path clip-path="url(#clip410)" d="M217.552 2345.69 L222.326 2337.42 L193.784 2320.94 L190.395 2330.98 L185.784 2328.32 L189.145 2318.33 L192.067 2313.27 L225.219 2332.41 L229.994 2324.14 L234.254 2326.6 L221.812 2348.15 L217.552 2345.69 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M239.466 2315.77 L234.856 2313.11 Q236.857 2311.73 238.462 2310.05 Q240.066 2308.37 241.151 2306.49 Q244.045 2301.48 242.206 2296.91 Q240.357 2292.3 233.708 2287.96 Q235.024 2290.66 234.889 2293.55 Q234.754 2296.45 233.191 2299.15 Q229.951 2304.77 224.673 2306.1 Q219.384 2307.39 213.495 2303.99 Q207.732 2300.66 206.216 2295.24 Q204.701 2289.82 207.97 2284.16 Q211.717 2277.67 218.672 2277.14 Q225.615 2276.57 235.087 2282.04 Q243.933 2287.15 246.79 2294.41 Q249.636 2301.63 245.542 2308.72 Q244.442 2310.63 242.938 2312.37 Q241.433 2314.1 239.466 2315.77 M229.42 2296.54 Q231.388 2293.14 230.2 2289.81 Q229.027 2286.46 224.968 2284.12 Q220.933 2281.79 217.446 2282.45 Q213.947 2283.07 211.98 2286.47 Q210.012 2289.88 211.21 2293.25 Q212.398 2296.57 216.432 2298.9 Q220.491 2301.24 223.965 2300.61 Q227.453 2299.95 229.42 2296.54 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M220.857 2263.4 L234.746 2239.34 L236.901 2240.59 L264.317 2274.53 L261.264 2279.81 L235.491 2247.89 L225.117 2265.86 L220.857 2263.4 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M266.541 2224.09 Q269.415 2220.9 273.043 2220.29 Q276.685 2219.65 280.293 2221.74 Q285.831 2224.94 286.664 2230.5 Q287.497 2236.05 283.446 2243.07 Q282.087 2245.43 280.164 2247.66 Q278.28 2249.88 275.82 2252 L270.933 2249.18 Q273.376 2247.78 275.45 2245.64 Q277.523 2243.49 279.1 2240.76 Q281.849 2236 281.401 2232.43 Q280.969 2228.84 277.385 2226.77 Q274.078 2224.87 270.878 2226.13 Q267.668 2227.35 265.28 2231.48 L262.763 2235.84 L258.603 2233.44 L261.236 2228.88 Q263.392 2225.14 263.057 2222.31 Q262.696 2219.46 259.889 2217.84 Q257.008 2216.18 254.293 2217.35 Q251.567 2218.48 249.368 2222.29 Q248.167 2224.37 247.244 2227.01 Q246.321 2229.65 245.624 2233.06 L241.113 2230.46 Q241.975 2227.11 243.021 2224.38 Q244.081 2221.61 245.369 2219.38 Q248.696 2213.62 253.266 2211.78 Q257.811 2209.93 262.271 2212.5 Q265.378 2214.3 266.506 2217.32 Q267.609 2220.33 266.541 2224.09 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M278.876 2212.1 L286.674 2198.59 L290.783 2200.97 L282.985 2214.47 L278.876 2212.1 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M279.572 2169.4 Q277.315 2173.31 280.031 2177.52 Q282.736 2181.69 290.454 2186.14 Q298.147 2190.58 303.135 2190.86 Q308.112 2191.09 310.369 2187.18 Q312.64 2183.25 309.935 2179.08 Q307.219 2174.87 299.526 2170.43 Q291.808 2165.97 286.831 2165.74 Q281.843 2165.47 279.572 2169.4 M275.562 2167.09 Q279.194 2160.8 286.09 2160.37 Q292.976 2159.9 302.448 2165.37 Q311.895 2170.82 314.958 2177.03 Q318.009 2183.21 314.378 2189.5 Q310.747 2195.79 303.861 2196.25 Q296.965 2196.68 287.518 2191.23 Q278.045 2185.76 274.994 2179.59 Q271.931 2173.38 275.562 2167.09 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M322.499 2163.91 L327.273 2155.64 L298.732 2139.17 L295.342 2149.2 L290.731 2146.54 L294.092 2136.55 L297.014 2131.49 L330.167 2150.63 L334.941 2142.36 L339.201 2144.82 L326.759 2166.37 L322.499 2163.91 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M327.269 2128.28 L335.067 2114.77 L339.177 2117.14 L331.379 2130.65 L327.269 2128.28 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M327.966 2085.58 Q325.709 2089.49 328.425 2093.7 Q331.13 2097.86 338.848 2102.32 Q346.541 2106.76 351.529 2107.04 Q356.506 2107.27 358.763 2103.36 Q361.034 2099.43 358.328 2095.26 Q355.612 2091.05 347.919 2086.61 Q340.201 2082.15 335.225 2081.92 Q330.237 2081.65 327.966 2085.58 M323.956 2083.27 Q327.588 2076.98 334.484 2076.55 Q341.37 2076.08 350.842 2081.55 Q360.289 2087 363.351 2093.21 Q366.403 2099.39 362.772 2105.68 Q359.141 2111.96 352.255 2112.43 Q345.358 2112.86 335.911 2107.41 Q326.439 2101.94 323.387 2095.77 Q320.325 2089.55 323.956 2083.27 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M370.893 2080.09 L375.667 2071.82 L347.125 2055.35 L343.736 2065.38 L339.125 2062.72 L342.486 2052.73 L345.408 2047.67 L378.561 2066.81 L383.335 2058.54 L387.595 2061 L375.153 2082.55 L370.893 2080.09 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M907.729 2345.69 L912.504 2337.42 L883.962 2320.94 L880.572 2330.98 L875.962 2328.32 L879.322 2318.33 L882.245 2313.27 L915.397 2332.41 L920.171 2324.14 L924.431 2326.6 L911.989 2348.15 L907.729 2345.69 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M929.644 2315.77 L925.033 2313.11 Q927.035 2311.73 928.64 2310.05 Q930.244 2308.37 931.329 2306.49 Q934.223 2301.48 932.384 2296.91 Q930.535 2292.3 923.886 2287.96 Q925.202 2290.66 925.067 2293.55 Q924.932 2296.45 923.369 2299.15 Q920.128 2304.77 914.85 2306.1 Q909.562 2307.39 903.673 2303.99 Q897.909 2300.66 896.394 2295.24 Q894.878 2289.82 898.148 2284.16 Q901.895 2277.67 908.849 2277.14 Q915.793 2276.57 925.265 2282.04 Q934.111 2287.15 936.967 2294.41 Q939.814 2301.63 935.719 2308.72 Q934.62 2310.63 933.115 2312.37 Q931.611 2314.1 929.644 2315.77 M919.598 2296.54 Q921.566 2293.14 920.378 2289.81 Q919.205 2286.46 915.146 2284.12 Q911.111 2281.79 907.623 2282.45 Q904.125 2283.07 902.157 2286.47 Q900.19 2289.88 901.388 2293.25 Q902.575 2296.57 906.61 2298.9 Q910.669 2301.24 914.143 2300.61 Q917.63 2299.95 919.598 2296.54 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M937.669 2262.64 Q935.585 2266.25 936.314 2269.44 Q937.057 2272.61 940.44 2274.56 Q943.823 2276.52 946.939 2275.58 Q950.069 2274.61 952.153 2271 Q954.236 2267.39 953.507 2264.2 Q952.753 2260.99 949.396 2259.05 Q946.013 2257.1 942.882 2258.07 Q939.767 2259.01 937.669 2262.64 M932.591 2266.46 Q929.909 2269.25 926.622 2269.79 Q923.351 2270.31 920.143 2268.46 Q915.658 2265.87 914.889 2261.18 Q914.135 2256.47 917.346 2250.91 Q920.573 2245.32 925.016 2243.64 Q929.46 2241.96 933.945 2244.55 Q937.153 2246.4 938.327 2249.52 Q939.515 2252.61 938.451 2256.31 Q941.415 2253.14 945.068 2252.54 Q948.735 2251.92 952.318 2253.99 Q957.756 2257.13 958.738 2262.14 Q959.735 2267.13 956.162 2273.32 Q952.588 2279.51 947.758 2281.16 Q942.941 2282.79 937.503 2279.65 Q933.92 2277.58 932.626 2274.09 Q931.331 2270.61 932.591 2266.46 M923.527 2263.7 Q926.434 2265.37 929.104 2264.51 Q931.789 2263.62 933.685 2260.34 Q935.565 2257.08 934.993 2254.31 Q934.434 2251.52 931.528 2249.84 Q928.621 2248.16 925.921 2249.07 Q923.237 2249.96 921.356 2253.22 Q919.461 2256.5 920.033 2259.27 Q920.62 2262.02 923.527 2263.7 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M953.925 2227.65 Q951.957 2231.06 953.13 2234.41 Q954.318 2237.74 958.377 2240.08 Q962.412 2242.41 965.91 2241.79 Q969.398 2241.13 971.365 2237.72 Q973.333 2234.32 972.146 2230.99 Q970.947 2227.63 966.913 2225.3 Q962.854 2222.95 959.366 2223.61 Q955.892 2224.25 953.925 2227.65 M943.864 2208.45 L948.475 2211.11 Q946.473 2212.49 944.869 2214.17 Q943.279 2215.83 942.179 2217.73 Q939.286 2222.74 941.135 2227.35 Q942.999 2231.94 949.623 2236.26 Q948.296 2233.53 948.431 2230.63 Q948.541 2227.72 950.089 2225.04 Q953.344 2219.4 958.658 2218.13 Q963.961 2216.81 969.85 2220.21 Q975.613 2223.54 977.129 2228.96 Q978.644 2234.38 975.375 2240.04 Q971.628 2246.53 964.684 2247.1 Q957.715 2247.65 948.268 2242.2 Q939.398 2237.08 936.566 2229.83 Q933.709 2222.57 937.803 2215.47 Q938.903 2213.57 940.393 2211.86 Q941.897 2210.12 943.864 2208.45 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M969.053 2212.1 L976.851 2198.59 L980.961 2200.97 L973.163 2214.47 L969.053 2212.1 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M969.75 2169.4 Q967.493 2173.31 970.209 2177.52 Q972.914 2181.69 980.632 2186.14 Q988.325 2190.58 993.313 2190.86 Q998.29 2191.09 1000.55 2187.18 Q1002.82 2183.25 1000.11 2179.08 Q997.396 2174.87 989.703 2170.43 Q981.985 2165.97 977.008 2165.74 Q972.021 2165.47 969.75 2169.4 M965.74 2167.09 Q969.372 2160.8 976.268 2160.37 Q983.154 2159.9 992.626 2165.37 Q1002.07 2170.82 1005.14 2177.03 Q1008.19 2183.21 1004.56 2189.5 Q1000.92 2195.79 994.039 2196.25 Q987.142 2196.68 977.695 2191.23 Q968.223 2185.76 965.171 2179.59 Q962.109 2173.38 965.74 2167.09 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M1012.68 2163.91 L1017.45 2155.64 L988.909 2139.17 L985.52 2149.2 L980.909 2146.54 L984.27 2136.55 L987.192 2131.49 L1020.34 2150.63 L1025.12 2142.36 L1029.38 2144.82 L1016.94 2166.37 L1012.68 2163.91 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M1017.45 2128.28 L1025.25 2114.77 L1029.35 2117.14 L1021.56 2130.65 L1017.45 2128.28 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M1018.14 2085.58 Q1015.89 2089.49 1018.6 2093.7 Q1021.31 2097.86 1029.03 2102.32 Q1036.72 2106.76 1041.71 2107.04 Q1046.68 2107.27 1048.94 2103.36 Q1051.21 2099.43 1048.51 2095.26 Q1045.79 2091.05 1038.1 2086.61 Q1030.38 2082.15 1025.4 2081.92 Q1020.41 2081.65 1018.14 2085.58 M1014.13 2083.27 Q1017.77 2076.98 1024.66 2076.55 Q1031.55 2076.08 1041.02 2081.55 Q1050.47 2087 1053.53 2093.21 Q1056.58 2099.39 1052.95 2105.68 Q1049.32 2111.96 1042.43 2112.43 Q1035.54 2112.86 1026.09 2107.41 Q1016.62 2101.94 1013.57 2095.77 Q1010.5 2089.55 1014.13 2083.27 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M1061.07 2080.09 L1065.84 2071.82 L1037.3 2055.35 L1033.91 2065.38 L1029.3 2062.72 L1032.66 2052.73 L1035.59 2047.67 L1068.74 2066.81 L1073.51 2058.54 L1077.77 2061 L1065.33 2082.55 L1061.07 2080.09 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M1597.91 2345.69 L1602.68 2337.42 L1574.14 2320.94 L1570.75 2330.98 L1566.14 2328.32 L1569.5 2318.33 L1572.42 2313.27 L1605.57 2332.41 L1610.35 2324.14 L1614.61 2326.6 L1602.17 2348.15 L1597.91 2345.69 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M1619.82 2315.77 L1615.21 2313.11 Q1617.21 2311.73 1618.82 2310.05 Q1620.42 2308.37 1621.51 2306.49 Q1624.4 2301.48 1622.56 2296.91 Q1620.71 2292.3 1614.06 2287.96 Q1615.38 2290.66 1615.24 2293.55 Q1615.11 2296.45 1613.55 2299.15 Q1610.31 2304.77 1605.03 2306.1 Q1599.74 2307.39 1593.85 2303.99 Q1588.09 2300.66 1586.57 2295.24 Q1585.06 2289.82 1588.33 2284.16 Q1592.07 2277.67 1599.03 2277.14 Q1605.97 2276.57 1615.44 2282.04 Q1624.29 2287.15 1627.15 2294.41 Q1629.99 2301.63 1625.9 2308.72 Q1624.8 2310.63 1623.29 2312.37 Q1621.79 2314.1 1619.82 2315.77 M1609.78 2296.54 Q1611.74 2293.14 1610.56 2289.81 Q1609.38 2286.46 1605.32 2284.12 Q1601.29 2281.79 1597.8 2282.45 Q1594.3 2283.07 1592.34 2286.47 Q1590.37 2289.88 1591.57 2293.25 Q1592.75 2296.57 1596.79 2298.9 Q1600.85 2301.24 1604.32 2300.61 Q1607.81 2299.95 1609.78 2296.54 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M1638.67 2283.12 L1634.06 2280.46 Q1636.06 2279.08 1637.67 2277.4 Q1639.27 2275.72 1640.36 2273.84 Q1643.25 2268.83 1641.41 2264.26 Q1639.56 2259.65 1632.91 2255.31 Q1634.23 2258.01 1634.1 2260.9 Q1633.96 2263.8 1632.4 2266.5 Q1629.16 2272.12 1623.88 2273.45 Q1618.59 2274.74 1612.7 2271.34 Q1606.94 2268.01 1605.42 2262.59 Q1603.91 2257.17 1607.18 2251.51 Q1610.92 2245.02 1617.88 2244.49 Q1624.82 2243.92 1634.29 2249.39 Q1643.14 2254.5 1646 2261.76 Q1648.84 2268.98 1644.75 2276.07 Q1643.65 2277.98 1642.14 2279.71 Q1640.64 2281.45 1638.67 2283.12 M1628.63 2263.89 Q1630.59 2260.48 1629.41 2257.16 Q1628.23 2253.81 1624.17 2251.46 Q1620.14 2249.14 1616.65 2249.79 Q1613.15 2250.41 1611.19 2253.82 Q1609.22 2257.23 1610.42 2260.6 Q1611.6 2263.92 1615.64 2266.25 Q1619.7 2268.59 1623.17 2267.96 Q1626.66 2267.3 1628.63 2263.89 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M1657.52 2250.47 L1652.91 2247.81 Q1654.92 2246.43 1656.52 2244.75 Q1658.12 2243.07 1659.21 2241.19 Q1662.1 2236.18 1660.26 2231.61 Q1658.41 2227 1651.77 2222.66 Q1653.08 2225.35 1652.95 2228.25 Q1652.81 2231.15 1651.25 2233.85 Q1648.01 2239.47 1642.73 2240.79 Q1637.44 2242.08 1631.55 2238.69 Q1625.79 2235.36 1624.27 2229.94 Q1622.76 2224.52 1626.03 2218.86 Q1629.78 2212.37 1636.73 2211.84 Q1643.67 2211.27 1653.15 2216.74 Q1661.99 2221.84 1664.85 2229.11 Q1667.69 2236.33 1663.6 2243.42 Q1662.5 2245.33 1661 2247.06 Q1659.49 2248.8 1657.52 2250.47 M1647.48 2231.24 Q1649.45 2227.83 1648.26 2224.51 Q1647.09 2221.16 1643.03 2218.81 Q1638.99 2216.48 1635.5 2217.14 Q1632.01 2217.76 1630.04 2221.17 Q1628.07 2224.58 1629.27 2227.94 Q1630.46 2231.27 1634.49 2233.6 Q1638.55 2235.94 1642.02 2235.31 Q1645.51 2234.65 1647.48 2231.24 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M1659.23 2212.1 L1667.03 2198.59 L1671.14 2200.97 L1663.34 2214.47 L1659.23 2212.1 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M1659.93 2169.4 Q1657.67 2173.31 1660.39 2177.52 Q1663.09 2181.69 1670.81 2186.14 Q1678.5 2190.58 1683.49 2190.86 Q1688.47 2191.09 1690.72 2187.18 Q1693 2183.25 1690.29 2179.08 Q1687.57 2174.87 1679.88 2170.43 Q1672.16 2165.97 1667.19 2165.74 Q1662.2 2165.47 1659.93 2169.4 M1655.92 2167.09 Q1659.55 2160.8 1666.45 2160.37 Q1673.33 2159.9 1682.8 2165.37 Q1692.25 2170.82 1695.31 2177.03 Q1698.36 2183.21 1694.73 2189.5 Q1691.1 2195.79 1684.22 2196.25 Q1677.32 2196.68 1667.87 2191.23 Q1658.4 2185.76 1655.35 2179.59 Q1652.29 2173.38 1655.92 2167.09 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M1702.85 2163.91 L1707.63 2155.64 L1679.09 2139.17 L1675.7 2149.2 L1671.09 2146.54 L1674.45 2136.55 L1677.37 2131.49 L1710.52 2150.63 L1715.3 2142.36 L1719.56 2144.82 L1707.11 2166.37 L1702.85 2163.91 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M1707.63 2128.28 L1715.42 2114.77 L1719.53 2117.14 L1711.73 2130.65 L1707.63 2128.28 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M1708.32 2085.58 Q1706.06 2089.49 1708.78 2093.7 Q1711.49 2097.86 1719.2 2102.32 Q1726.9 2106.76 1731.88 2107.04 Q1736.86 2107.27 1739.12 2103.36 Q1741.39 2099.43 1738.68 2095.26 Q1735.97 2091.05 1728.27 2086.61 Q1720.56 2082.15 1715.58 2081.92 Q1710.59 2081.65 1708.32 2085.58 M1704.31 2083.27 Q1707.94 2076.98 1714.84 2076.55 Q1721.73 2076.08 1731.2 2081.55 Q1740.64 2087 1743.71 2093.21 Q1746.76 2099.39 1743.13 2105.68 Q1739.5 2111.96 1732.61 2112.43 Q1725.71 2112.86 1716.27 2107.41 Q1706.79 2101.94 1703.74 2095.77 Q1700.68 2089.55 1704.31 2083.27 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M1751.25 2080.09 L1756.02 2071.82 L1727.48 2055.35 L1724.09 2065.38 L1719.48 2062.72 L1722.84 2052.73 L1725.76 2047.67 L1758.92 2066.81 L1763.69 2058.54 L1767.95 2061 L1755.51 2082.55 L1751.25 2080.09 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M2290.1 2342.2 L2300.3 2324.54 L2304.56 2327 L2290.84 2350.75 L2286.58 2348.29 Q2285.26 2343.69 2283.12 2335.84 Q2280.96 2327.94 2280.24 2325.82 Q2278.91 2321.82 2277.58 2319.78 Q2276.24 2317.7 2274.41 2316.65 Q2271.43 2314.92 2268.34 2315.94 Q2265.26 2316.94 2263.32 2320.3 Q2261.94 2322.68 2261.24 2325.81 Q2260.55 2328.92 2260.49 2332.9 L2255.38 2329.94 Q2255.91 2326.18 2256.82 2323.16 Q2257.73 2320.14 2259 2317.94 Q2262.36 2312.12 2267.26 2310.34 Q2272.16 2308.56 2277.03 2311.37 Q2279.33 2312.7 2280.9 2314.78 Q2282.47 2316.82 2283.96 2320.72 Q2284.32 2321.76 2285.87 2327.13 Q2287.39 2332.49 2290.1 2342.2 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M2282.86 2285.87 Q2280.6 2289.78 2283.32 2293.99 Q2286.02 2298.16 2293.74 2302.61 Q2301.44 2307.05 2306.42 2307.33 Q2311.4 2307.56 2313.66 2303.65 Q2315.93 2299.72 2313.22 2295.55 Q2310.51 2291.34 2302.81 2286.9 Q2295.1 2282.45 2290.12 2282.21 Q2285.13 2281.94 2282.86 2285.87 M2278.85 2283.56 Q2282.48 2277.27 2289.38 2276.84 Q2296.26 2276.37 2305.74 2281.84 Q2315.18 2287.29 2318.25 2293.51 Q2321.3 2299.68 2317.67 2305.97 Q2314.04 2312.26 2307.15 2312.73 Q2300.25 2313.15 2290.81 2307.7 Q2281.33 2302.23 2278.28 2296.06 Q2275.22 2289.85 2278.85 2283.56 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M2325.79 2280.39 L2330.56 2272.12 L2302.02 2255.64 L2298.63 2265.68 L2294.02 2263.01 L2297.38 2253.03 L2300.3 2247.96 L2333.46 2267.1 L2338.23 2258.84 L2342.49 2261.29 L2330.05 2282.85 L2325.79 2280.39 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M2346.65 2244.25 L2356.85 2226.59 L2361.11 2229.04 L2347.39 2252.8 L2343.13 2250.34 Q2341.82 2245.74 2339.67 2237.88 Q2337.51 2229.99 2336.8 2227.87 Q2335.47 2223.86 2334.14 2221.83 Q2332.8 2219.75 2330.97 2218.69 Q2327.99 2216.97 2324.89 2217.99 Q2321.81 2218.99 2319.87 2222.34 Q2318.5 2224.72 2317.79 2227.86 Q2317.1 2230.97 2317.04 2234.94 L2311.93 2231.99 Q2312.47 2228.22 2313.37 2225.21 Q2314.28 2222.19 2315.55 2219.98 Q2318.91 2214.17 2323.81 2212.39 Q2328.72 2210.61 2333.58 2213.42 Q2335.88 2214.75 2337.46 2216.83 Q2339.02 2218.86 2340.51 2222.77 Q2340.88 2223.81 2342.42 2229.18 Q2343.94 2234.53 2346.65 2244.25 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M2349.41 2212.1 L2357.21 2198.59 L2361.32 2200.97 L2353.52 2214.47 L2349.41 2212.1 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M2350.11 2169.4 Q2347.85 2173.31 2350.56 2177.52 Q2353.27 2181.69 2360.99 2186.14 Q2368.68 2190.58 2373.67 2190.86 Q2378.65 2191.09 2380.9 2187.18 Q2383.17 2183.25 2380.47 2179.08 Q2377.75 2174.87 2370.06 2170.43 Q2362.34 2165.97 2357.36 2165.74 Q2352.38 2165.47 2350.11 2169.4 M2346.1 2167.09 Q2349.73 2160.8 2356.62 2160.37 Q2363.51 2159.9 2372.98 2165.37 Q2382.43 2170.82 2385.49 2177.03 Q2388.54 2183.21 2384.91 2189.5 Q2381.28 2195.79 2374.39 2196.25 Q2367.5 2196.68 2358.05 2191.23 Q2348.58 2185.76 2345.53 2179.59 Q2342.46 2173.38 2346.1 2167.09 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M2393.03 2163.91 L2397.81 2155.64 L2369.26 2139.17 L2365.88 2149.2 L2361.26 2146.54 L2364.63 2136.55 L2367.55 2131.49 L2400.7 2150.63 L2405.47 2142.36 L2409.73 2144.82 L2397.29 2166.37 L2393.03 2163.91 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M2397.8 2128.28 L2405.6 2114.77 L2409.71 2117.14 L2401.91 2130.65 L2397.8 2128.28 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M2398.5 2085.58 Q2396.24 2089.49 2398.96 2093.7 Q2401.66 2097.86 2409.38 2102.32 Q2417.07 2106.76 2422.06 2107.04 Q2427.04 2107.27 2429.3 2103.36 Q2431.57 2099.43 2428.86 2095.26 Q2426.15 2091.05 2418.45 2086.61 Q2410.73 2082.15 2405.76 2081.92 Q2400.77 2081.65 2398.5 2085.58 M2394.49 2083.27 Q2398.12 2076.98 2405.02 2076.55 Q2411.9 2076.08 2421.38 2081.55 Q2430.82 2087 2433.88 2093.21 Q2436.94 2099.39 2433.31 2105.68 Q2429.67 2111.96 2422.79 2112.43 Q2415.89 2112.86 2406.44 2107.41 Q2396.97 2101.94 2393.92 2095.77 Q2390.86 2089.55 2394.49 2083.27 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M2441.43 2080.09 L2446.2 2071.82 L2417.66 2055.35 L2414.27 2065.38 L2409.66 2062.72 L2413.02 2052.73 L2415.94 2047.67 L2449.09 2066.81 L2453.87 2058.54 L2458.13 2061 L2445.69 2082.55 L2441.43 2080.09 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M2980.42 2342.2 L2990.62 2324.54 L2994.88 2327 L2981.16 2350.75 L2976.9 2348.29 Q2975.59 2343.69 2973.44 2335.84 Q2971.28 2327.94 2970.56 2325.82 Q2969.24 2321.82 2967.91 2319.78 Q2966.57 2317.7 2964.74 2316.65 Q2961.76 2314.92 2958.66 2315.94 Q2955.58 2316.94 2953.64 2320.3 Q2952.27 2322.68 2951.56 2325.81 Q2950.87 2328.92 2950.81 2332.9 L2945.7 2329.94 Q2946.24 2326.18 2947.14 2323.16 Q2948.05 2320.14 2949.32 2317.94 Q2952.68 2312.12 2957.58 2310.34 Q2962.49 2308.56 2967.35 2311.37 Q2969.65 2312.7 2971.23 2314.78 Q2972.79 2316.82 2974.28 2320.72 Q2974.65 2321.76 2976.19 2327.13 Q2977.71 2332.49 2980.42 2342.2 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M2973.18 2285.87 Q2970.93 2289.78 2973.64 2293.99 Q2976.35 2298.16 2984.07 2302.61 Q2991.76 2307.05 2996.75 2307.33 Q3001.72 2307.56 3003.98 2303.65 Q3006.25 2299.72 3003.55 2295.55 Q3000.83 2291.34 2993.14 2286.9 Q2985.42 2282.45 2980.44 2282.21 Q2975.45 2281.94 2973.18 2285.87 M2969.17 2283.56 Q2972.81 2277.27 2979.7 2276.84 Q2986.59 2276.37 2996.06 2281.84 Q3005.51 2287.29 3008.57 2293.51 Q3011.62 2299.68 3007.99 2305.97 Q3004.36 2312.26 2997.47 2312.73 Q2990.58 2313.15 2981.13 2307.7 Q2971.66 2302.23 2968.61 2296.06 Q2965.54 2289.85 2969.17 2283.56 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M3018.12 2276.9 L3028.32 2259.24 L3032.58 2261.7 L3018.87 2285.45 L3014.61 2282.99 Q3013.29 2278.39 3011.14 2270.53 Q3008.98 2262.64 3008.27 2260.52 Q3006.94 2256.51 3005.61 2254.48 Q3004.27 2252.4 3002.44 2251.34 Q2999.46 2249.62 2996.36 2250.64 Q2993.28 2251.64 2991.34 2254.99 Q2989.97 2257.38 2989.26 2260.51 Q2988.57 2263.62 2988.51 2267.59 L2983.4 2264.64 Q2983.94 2260.88 2984.85 2257.86 Q2985.75 2254.84 2987.03 2252.64 Q2990.38 2246.82 2995.29 2245.04 Q3000.19 2243.26 3005.05 2246.07 Q3007.36 2247.4 3008.93 2249.48 Q3010.49 2251.52 3011.98 2255.42 Q3012.35 2256.46 3013.89 2261.83 Q3015.41 2267.19 3018.12 2276.9 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M3001.33 2229.42 L3012.8 2209.55 L3017.06 2212.01 L3008.27 2227.24 L3017.44 2232.54 Q3017.7 2231.22 3018.16 2230.02 Q3018.6 2228.8 3019.23 2227.7 Q3022.85 2221.43 3028.4 2219.75 Q3033.94 2218.08 3039.81 2221.46 Q3045.84 2224.95 3047.03 2230.65 Q3048.19 2236.33 3044.25 2243.17 Q3042.89 2245.53 3041.07 2247.75 Q3039.26 2249.95 3036.96 2252.09 L3031.87 2249.16 Q3034.4 2247.61 3036.39 2245.55 Q3038.38 2243.49 3039.93 2240.81 Q3042.43 2236.48 3041.61 2232.63 Q3040.79 2228.78 3036.88 2226.52 Q3032.97 2224.27 3029.23 2225.48 Q3025.49 2226.7 3022.99 2231.03 Q3021.82 2233.06 3021.1 2235.35 Q3020.39 2237.62 3020.13 2240.27 L3001.33 2229.42 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M3039.73 2212.1 L3047.53 2198.59 L3051.64 2200.97 L3043.84 2214.47 L3039.73 2212.1 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M3040.43 2169.4 Q3038.17 2173.31 3040.89 2177.52 Q3043.59 2181.69 3051.31 2186.14 Q3059 2190.58 3063.99 2190.86 Q3068.97 2191.09 3071.23 2187.18 Q3073.5 2183.25 3070.79 2179.08 Q3068.08 2174.87 3060.38 2170.43 Q3052.66 2165.97 3047.69 2165.74 Q3042.7 2165.47 3040.43 2169.4 M3036.42 2167.09 Q3040.05 2160.8 3046.95 2160.37 Q3053.83 2159.9 3063.3 2165.37 Q3072.75 2170.82 3075.81 2177.03 Q3078.87 2183.21 3075.23 2189.5 Q3071.6 2195.79 3064.72 2196.25 Q3057.82 2196.68 3048.37 2191.23 Q3038.9 2185.76 3035.85 2179.59 Q3032.79 2173.38 3036.42 2167.09 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M3083.36 2163.91 L3088.13 2155.64 L3059.59 2139.17 L3056.2 2149.2 L3051.59 2146.54 L3054.95 2136.55 L3057.87 2131.49 L3091.02 2150.63 L3095.8 2142.36 L3100.06 2144.82 L3087.62 2166.37 L3083.36 2163.91 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M3088.13 2128.28 L3095.92 2114.77 L3100.03 2117.14 L3092.24 2130.65 L3088.13 2128.28 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M3088.82 2085.58 Q3086.57 2089.49 3089.28 2093.7 Q3091.99 2097.86 3099.7 2102.32 Q3107.4 2106.76 3112.39 2107.04 Q3117.36 2107.27 3119.62 2103.36 Q3121.89 2099.43 3119.18 2095.26 Q3116.47 2091.05 3108.78 2086.61 Q3101.06 2082.15 3096.08 2081.92 Q3091.09 2081.65 3088.82 2085.58 M3084.81 2083.27 Q3088.44 2076.98 3095.34 2076.55 Q3102.23 2076.08 3111.7 2081.55 Q3121.15 2087 3124.21 2093.21 Q3127.26 2099.39 3123.63 2105.68 Q3120 2111.96 3113.11 2112.43 Q3106.21 2112.86 3096.77 2107.41 Q3087.3 2101.94 3084.24 2095.77 Q3081.18 2089.55 3084.81 2083.27 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M3131.75 2080.09 L3136.52 2071.82 L3107.98 2055.35 L3104.59 2065.38 L3099.98 2062.72 L3103.34 2052.73 L3106.26 2047.67 L3139.42 2066.81 L3144.19 2058.54 L3148.45 2061 L3136.01 2082.55 L3131.75 2080.09 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><polyline clip-path="url(#clip412)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="
  346.895,2007.89 3152.76,2007.89 
  "/>
<polyline clip-path="url(#clip412)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="
  346.895,1607.44 3152.76,1607.44 
  "/>
<polyline clip-path="url(#clip412)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="
  346.895,1206.99 3152.76,1206.99 
  "/>
<polyline clip-path="url(#clip412)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="
  346.895,806.542 3152.76,806.542 
  "/>
<polyline clip-path="url(#clip412)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="
  346.895,406.093 3152.76,406.093 
  "/>
<polyline clip-path="url(#clip410)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="
  346.895,2008.4 346.895,47.2441 
  "/>
<polyline clip-path="url(#clip410)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="
  346.895,2007.89 365.793,2007.89 
  "/>
<polyline clip-path="url(#clip410)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="
  346.895,1607.44 365.793,1607.44 
  "/>
<polyline clip-path="url(#clip410)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="
  346.895,1206.99 365.793,1206.99 
  "/>
<polyline clip-path="url(#clip410)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="
  346.895,806.542 365.793,806.542 
  "/>
<polyline clip-path="url(#clip410)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="
  346.895,406.093 365.793,406.093 
  "/>
<path clip-path="url(#clip410)" d="M184.052 2008.45 L221.147 2008.45 L221.147 2013.37 L184.052 2013.37 L184.052 2008.45 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M249.822 1991.38 L235.065 2014.44 L249.822 2014.44 L249.822 1991.38 M248.288 1986.29 L255.638 1986.29 L255.638 2014.44 L261.801 2014.44 L261.801 2019.3 L255.638 2019.3 L255.638 2029.49 L249.822 2029.49 L249.822 2019.3 L230.319 2019.3 L230.319 2013.66 L248.288 1986.29 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M283.965 1990.14 Q279.451 1990.14 277.165 1994.59 Q274.908 1999.02 274.908 2007.93 Q274.908 2016.82 277.165 2021.27 Q279.451 2025.7 283.965 2025.7 Q288.508 2025.7 290.765 2021.27 Q293.051 2016.82 293.051 2007.93 Q293.051 1999.02 290.765 1994.59 Q288.508 1990.14 283.965 1990.14 M283.965 1985.51 Q291.228 1985.51 295.047 1991.27 Q298.895 1996.99 298.895 2007.93 Q298.895 2018.84 295.047 2024.6 Q291.228 2030.33 283.965 2030.33 Q276.702 2030.33 272.854 2024.6 Q269.034 2018.84 269.034 2007.93 Q269.034 1996.99 272.854 1991.27 Q276.702 1985.51 283.965 1985.51 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M184.052 1608 L221.147 1608 L221.147 1612.92 L184.052 1612.92 L184.052 1608 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M238.797 1624.12 L259.197 1624.12 L259.197 1629.04 L231.766 1629.04 L231.766 1624.12 Q235.094 1620.68 240.823 1614.89 Q246.581 1609.07 248.057 1607.4 Q250.863 1604.24 251.963 1602.07 Q253.091 1599.87 253.091 1597.76 Q253.091 1594.32 250.661 1592.15 Q248.259 1589.98 244.382 1589.98 Q241.633 1589.98 238.566 1590.93 Q235.528 1591.89 232.055 1593.83 L232.055 1587.92 Q235.586 1586.51 238.653 1585.78 Q241.72 1585.06 244.266 1585.06 Q250.979 1585.06 254.972 1588.41 Q258.965 1591.77 258.965 1597.38 Q258.965 1600.05 257.952 1602.45 Q256.969 1604.82 254.335 1608.06 Q253.612 1608.9 249.735 1612.92 Q245.858 1616.92 238.797 1624.12 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M283.965 1589.69 Q279.451 1589.69 277.165 1594.14 Q274.908 1598.57 274.908 1607.48 Q274.908 1616.37 277.165 1620.82 Q279.451 1625.25 283.965 1625.25 Q288.508 1625.25 290.765 1620.82 Q293.051 1616.37 293.051 1607.48 Q293.051 1598.57 290.765 1594.14 Q288.508 1589.69 283.965 1589.69 M283.965 1585.06 Q291.228 1585.06 295.047 1590.82 Q298.895 1596.55 298.895 1607.48 Q298.895 1618.39 295.047 1624.15 Q291.228 1629.88 283.965 1629.88 Q276.702 1629.88 272.854 1624.15 Q269.034 1618.39 269.034 1607.48 Q269.034 1596.55 272.854 1590.82 Q276.702 1585.06 283.965 1585.06 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M283.965 1189.24 Q279.451 1189.24 277.165 1193.7 Q274.908 1198.12 274.908 1207.03 Q274.908 1215.92 277.165 1220.37 Q279.451 1224.8 283.965 1224.8 Q288.508 1224.8 290.765 1220.37 Q293.051 1215.92 293.051 1207.03 Q293.051 1198.12 290.765 1193.7 Q288.508 1189.24 283.965 1189.24 M283.965 1184.61 Q291.228 1184.61 295.047 1190.37 Q298.895 1196.1 298.895 1207.03 Q298.895 1217.94 295.047 1223.7 Q291.228 1229.43 283.965 1229.43 Q276.702 1229.43 272.854 1223.7 Q269.034 1217.94 269.034 1207.03 Q269.034 1196.1 272.854 1190.37 Q276.702 1184.61 283.965 1184.61 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M238.797 823.223 L259.197 823.223 L259.197 828.142 L231.766 828.142 L231.766 823.223 Q235.094 819.78 240.823 813.993 Q246.581 808.177 248.057 806.499 Q250.863 803.345 251.963 801.175 Q253.091 798.976 253.091 796.863 Q253.091 793.42 250.661 791.25 Q248.259 789.08 244.382 789.08 Q241.633 789.08 238.566 790.035 Q235.528 790.99 232.055 792.928 L232.055 787.025 Q235.586 785.608 238.653 784.884 Q241.72 784.161 244.266 784.161 Q250.979 784.161 254.972 787.517 Q258.965 790.874 258.965 796.487 Q258.965 799.149 257.952 801.551 Q256.969 803.923 254.335 807.164 Q253.612 808.003 249.735 812.025 Q245.858 816.018 238.797 823.223 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M283.965 788.79 Q279.451 788.79 277.165 793.246 Q274.908 797.674 274.908 806.586 Q274.908 815.469 277.165 819.925 Q279.451 824.352 283.965 824.352 Q288.508 824.352 290.765 819.925 Q293.051 815.469 293.051 806.586 Q293.051 797.674 290.765 793.246 Q288.508 788.79 283.965 788.79 M283.965 784.161 Q291.228 784.161 295.047 789.919 Q298.895 795.648 298.895 806.586 Q298.895 817.494 295.047 823.252 Q291.228 828.981 283.965 828.981 Q276.702 828.981 272.854 823.252 Q269.034 817.494 269.034 806.586 Q269.034 795.648 272.854 789.919 Q276.702 784.161 283.965 784.161 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M249.822 389.586 L235.065 412.647 L249.822 412.647 L249.822 389.586 M248.288 384.493 L255.638 384.493 L255.638 412.647 L261.801 412.647 L261.801 417.508 L255.638 417.508 L255.638 427.693 L249.822 427.693 L249.822 417.508 L230.319 417.508 L230.319 411.866 L248.288 384.493 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M283.965 388.342 Q279.451 388.342 277.165 392.798 Q274.908 397.225 274.908 406.137 Q274.908 415.02 277.165 419.476 Q279.451 423.903 283.965 423.903 Q288.508 423.903 290.765 419.476 Q293.051 415.02 293.051 406.137 Q293.051 397.225 290.765 392.798 Q288.508 388.342 283.965 388.342 M283.965 383.712 Q291.228 383.712 295.047 389.47 Q298.895 395.199 298.895 406.137 Q298.895 417.045 295.047 422.803 Q291.228 428.532 283.965 428.532 Q276.702 428.532 272.854 422.803 Q269.034 417.045 269.034 406.137 Q269.034 395.199 272.854 389.47 Q276.702 383.712 283.965 383.712 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M74.4241 1636.57 L114.736 1636.57 L114.736 1628.1 Q114.736 1617.37 109.875 1612.4 Q105.014 1607.4 94.5282 1607.4 Q84.1116 1607.4 79.2852 1612.4 Q74.4241 1617.37 74.4241 1628.1 L74.4241 1636.57 M68.6603 1643.58 L68.6603 1629.17 Q68.6603 1614.1 74.945 1607.06 Q81.1949 1600.01 94.5282 1600.01 Q107.931 1600.01 114.216 1607.09 Q120.5 1614.17 120.5 1629.17 L120.5 1643.58 L68.6603 1643.58 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M81.6116 1589.1 L81.6116 1582.72 L120.5 1582.72 L120.5 1589.1 L81.6116 1589.1 M66.4728 1589.1 L66.4728 1582.72 L74.563 1582.72 L74.563 1589.1 L66.4728 1589.1 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M66.4728 1549.66 L71.7853 1549.66 L71.7853 1555.77 Q71.7853 1559.21 73.1741 1560.56 Q74.563 1561.88 78.1741 1561.88 L81.6116 1561.88 L81.6116 1551.36 L86.5769 1551.36 L86.5769 1561.88 L120.5 1561.88 L120.5 1568.31 L86.5769 1568.31 L86.5769 1574.42 L81.6116 1574.42 L81.6116 1568.31 L78.9033 1568.31 Q72.4103 1568.31 69.4589 1565.29 Q66.4728 1562.26 66.4728 1555.7 L66.4728 1549.66 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M66.4728 1524.63 L71.7853 1524.63 L71.7853 1530.74 Q71.7853 1534.17 73.1741 1535.53 Q74.563 1536.85 78.1741 1536.85 L81.6116 1536.85 L81.6116 1526.33 L86.5769 1526.33 L86.5769 1536.85 L120.5 1536.85 L120.5 1543.27 L86.5769 1543.27 L86.5769 1549.38 L81.6116 1549.38 L81.6116 1543.27 L78.9033 1543.27 Q72.4103 1543.27 69.4589 1540.25 Q66.4728 1537.23 66.4728 1530.67 L66.4728 1524.63 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M99.4587 1486.02 L102.584 1486.02 L102.584 1515.39 Q109.181 1514.97 112.653 1511.43 Q116.091 1507.86 116.091 1501.5 Q116.091 1497.82 115.188 1494.38 Q114.285 1490.91 112.479 1487.51 L118.521 1487.51 Q119.979 1490.95 120.743 1494.56 Q121.507 1498.17 121.507 1501.88 Q121.507 1511.19 116.091 1516.64 Q110.674 1522.06 101.438 1522.06 Q91.8893 1522.06 86.2991 1516.92 Q80.6741 1511.74 80.6741 1502.99 Q80.6741 1495.15 85.7435 1490.6 Q90.7782 1486.02 99.4587 1486.02 M97.5837 1492.4 Q92.3407 1492.47 89.2157 1495.36 Q86.0907 1498.2 86.0907 1502.92 Q86.0907 1508.27 89.1116 1511.5 Q92.1324 1514.7 97.6185 1515.18 L97.5837 1492.4 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M87.5838 1452.99 Q86.9588 1454.07 86.681 1455.36 Q86.3685 1456.61 86.3685 1458.13 Q86.3685 1463.55 89.9102 1466.47 Q93.4171 1469.35 100.014 1469.35 L120.5 1469.35 L120.5 1475.77 L81.6116 1475.77 L81.6116 1469.35 L87.6532 1469.35 Q84.1116 1467.33 82.4102 1464.11 Q80.6741 1460.88 80.6741 1456.26 Q80.6741 1455.6 80.7783 1454.8 Q80.8477 1454 81.0213 1453.03 L87.5838 1452.99 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M99.4587 1414.59 L102.584 1414.59 L102.584 1443.97 Q109.181 1443.55 112.653 1440.01 Q116.091 1436.43 116.091 1430.08 Q116.091 1426.4 115.188 1422.96 Q114.285 1419.49 112.479 1416.09 L118.521 1416.09 Q119.979 1419.52 120.743 1423.13 Q121.507 1426.74 121.507 1430.46 Q121.507 1439.77 116.091 1445.22 Q110.674 1450.63 101.438 1450.63 Q91.8893 1450.63 86.2991 1445.49 Q80.6741 1440.32 80.6741 1431.57 Q80.6741 1423.72 85.7435 1419.18 Q90.7782 1414.59 99.4587 1414.59 M97.5837 1420.98 Q92.3407 1421.05 89.2157 1423.93 Q86.0907 1426.78 86.0907 1431.5 Q86.0907 1436.85 89.1116 1440.08 Q92.1324 1443.27 97.6185 1443.76 L97.5837 1420.98 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M97.0282 1371.78 L120.5 1371.78 L120.5 1378.17 L97.2365 1378.17 Q91.7157 1378.17 88.9727 1380.32 Q86.2296 1382.47 86.2296 1386.78 Q86.2296 1391.95 89.5282 1394.94 Q92.8268 1397.93 98.5212 1397.93 L120.5 1397.93 L120.5 1404.35 L81.6116 1404.35 L81.6116 1397.93 L87.6532 1397.93 Q84.1463 1395.63 82.4102 1392.54 Q80.6741 1389.42 80.6741 1385.36 Q80.6741 1378.65 84.8408 1375.22 Q88.9727 1371.78 97.0282 1371.78 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M83.1047 1331.05 L89.0768 1331.05 Q87.5838 1333.76 86.8546 1336.5 Q86.0907 1339.21 86.0907 1341.99 Q86.0907 1348.2 90.0491 1351.64 Q93.9726 1355.08 101.091 1355.08 Q108.209 1355.08 112.167 1351.64 Q116.091 1348.2 116.091 1341.99 Q116.091 1339.21 115.361 1336.5 Q114.598 1333.76 113.104 1331.05 L119.007 1331.05 Q120.257 1333.72 120.882 1336.61 Q121.507 1339.45 121.507 1342.68 Q121.507 1351.47 115.986 1356.64 Q110.466 1361.81 101.091 1361.81 Q91.5768 1361.81 86.1255 1356.61 Q80.6741 1351.36 80.6741 1342.27 Q80.6741 1339.31 81.2991 1336.5 Q81.8894 1333.69 83.1047 1331.05 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M99.4587 1286.68 L102.584 1286.68 L102.584 1316.05 Q109.181 1315.63 112.653 1312.09 Q116.091 1308.52 116.091 1302.16 Q116.091 1298.48 115.188 1295.04 Q114.285 1291.57 112.479 1288.17 L118.521 1288.17 Q119.979 1291.61 120.743 1295.22 Q121.507 1298.83 121.507 1302.54 Q121.507 1311.85 116.091 1317.3 Q110.674 1322.72 101.438 1322.72 Q91.8893 1322.72 86.2991 1317.58 Q80.6741 1312.41 80.6741 1303.66 Q80.6741 1295.81 85.7435 1291.26 Q90.7782 1286.68 99.4587 1286.68 M97.5837 1293.06 Q92.3407 1293.13 89.2157 1296.02 Q86.0907 1298.86 86.0907 1303.59 Q86.0907 1308.93 89.1116 1312.16 Q92.1324 1315.36 97.6185 1315.84 L97.5837 1293.06 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M81.6116 1253.59 L81.6116 1247.2 L120.5 1247.2 L120.5 1253.59 L81.6116 1253.59 M66.4728 1253.59 L66.4728 1247.2 L74.563 1247.2 L74.563 1253.59 L66.4728 1253.59 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M97.0282 1201.5 L120.5 1201.5 L120.5 1207.89 L97.2365 1207.89 Q91.7157 1207.89 88.9727 1210.04 Q86.2296 1212.2 86.2296 1216.5 Q86.2296 1221.68 89.5282 1224.66 Q92.8268 1227.65 98.5212 1227.65 L120.5 1227.65 L120.5 1234.07 L81.6116 1234.07 L81.6116 1227.65 L87.6532 1227.65 Q84.1463 1225.36 82.4102 1222.27 Q80.6741 1219.14 80.6741 1215.08 Q80.6741 1208.38 84.8408 1204.94 Q88.9727 1201.5 97.0282 1201.5 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M68.6603 1173.07 L68.6603 1129.21 L74.563 1129.21 L74.563 1147.61 L120.5 1147.61 L120.5 1154.66 L74.563 1154.66 L74.563 1173.07 L68.6603 1173.07 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M81.6116 1124.91 L81.6116 1118.52 L120.5 1118.52 L120.5 1124.91 L81.6116 1124.91 M66.4728 1124.91 L66.4728 1118.52 L74.563 1118.52 L74.563 1124.91 L66.4728 1124.91 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M89.0768 1074.87 Q84.7713 1072.48 82.7227 1069.14 Q80.6741 1065.81 80.6741 1061.3 Q80.6741 1055.22 84.9449 1051.92 Q89.181 1048.62 97.0282 1048.62 L120.5 1048.62 L120.5 1055.05 L97.2365 1055.05 Q91.6463 1055.05 88.938 1057.02 Q86.2296 1059 86.2296 1063.07 Q86.2296 1068.03 89.5282 1070.91 Q92.8268 1073.8 98.5212 1073.8 L120.5 1073.8 L120.5 1080.22 L97.2365 1080.22 Q91.6115 1080.22 88.938 1082.2 Q86.2296 1084.18 86.2296 1088.31 Q86.2296 1093.2 89.5629 1096.09 Q92.8615 1098.97 98.5212 1098.97 L120.5 1098.97 L120.5 1105.39 L81.6116 1105.39 L81.6116 1098.97 L87.6532 1098.97 Q84.0769 1096.78 82.3755 1093.73 Q80.6741 1090.67 80.6741 1086.47 Q80.6741 1082.23 82.8269 1079.28 Q84.9796 1076.3 89.0768 1074.87 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M99.4587 1002.62 L102.584 1002.62 L102.584 1031.99 Q109.181 1031.57 112.653 1028.03 Q116.091 1024.46 116.091 1018.1 Q116.091 1014.42 115.188 1010.98 Q114.285 1007.51 112.479 1004.11 L118.521 1004.11 Q119.979 1007.55 120.743 1011.16 Q121.507 1014.77 121.507 1018.48 Q121.507 1027.79 116.091 1033.24 Q110.674 1038.66 101.438 1038.66 Q91.8893 1038.66 86.2991 1033.52 Q80.6741 1028.34 80.6741 1019.59 Q80.6741 1011.75 85.7435 1007.2 Q90.7782 1002.62 99.4587 1002.62 M97.5837 1009 Q92.3407 1009.07 89.2157 1011.96 Q86.0907 1014.8 86.0907 1019.52 Q86.0907 1024.87 89.1116 1028.1 Q92.1324 1031.3 97.6185 1031.78 L97.5837 1009 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M70.57 963.206 L81.6116 963.206 L81.6116 950.046 L86.5769 950.046 L86.5769 963.206 L107.688 963.206 Q112.445 963.206 113.799 961.921 Q115.153 960.601 115.153 956.608 L115.153 950.046 L120.5 950.046 L120.5 956.608 Q120.5 964.004 117.757 966.817 Q114.979 969.629 107.688 969.629 L86.5769 969.629 L86.5769 974.317 L81.6116 974.317 L81.6116 969.629 L70.57 969.629 L70.57 963.206 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M86.0907 926.574 Q86.0907 931.713 90.1185 934.699 Q94.1115 937.685 101.091 937.685 Q108.07 937.685 112.098 934.733 Q116.091 931.747 116.091 926.574 Q116.091 921.47 112.063 918.484 Q108.035 915.497 101.091 915.497 Q94.181 915.497 90.1532 918.484 Q86.0907 921.47 86.0907 926.574 M80.6741 926.574 Q80.6741 918.241 86.0907 913.484 Q91.5074 908.727 101.091 908.727 Q110.639 908.727 116.091 913.484 Q121.507 918.241 121.507 926.574 Q121.507 934.942 116.091 939.699 Q110.639 944.421 101.091 944.421 Q91.5074 944.421 86.0907 939.699 Q80.6741 934.942 80.6741 926.574 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M68.6603 882.442 L68.6603 838.588 L74.563 838.588 L74.563 856.991 L120.5 856.991 L120.5 864.039 L74.563 864.039 L74.563 882.442 L68.6603 882.442 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M75.57 820.012 L101.368 829.526 L101.368 810.463 L75.57 820.012 M68.6603 823.97 L68.6603 816.019 L120.5 796.262 L120.5 803.554 L107.202 808.276 L107.202 831.644 L120.5 836.366 L120.5 843.762 L68.6603 823.97 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M68.6603 788.693 L68.6603 781.679 L120.5 781.679 L120.5 788.693 L68.6603 788.693 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M66.5422 730.047 Q74.5283 734.7 82.3408 736.957 Q90.1532 739.214 98.174 739.214 Q106.195 739.214 114.077 736.957 Q121.924 734.665 129.875 730.047 L129.875 735.603 Q121.716 740.811 113.834 743.415 Q105.952 745.984 98.174 745.984 Q90.431 745.984 82.5838 743.415 Q74.7366 740.846 66.5422 735.603 L66.5422 730.047 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M82.7574 692.86 L88.7991 692.86 Q87.4102 695.568 86.7157 698.485 Q86.0213 701.401 86.0213 704.526 Q86.0213 709.283 87.4796 711.679 Q88.938 714.04 91.8546 714.04 Q94.0768 714.04 95.3615 712.339 Q96.6115 710.637 97.7573 705.499 L98.2435 703.311 Q99.7018 696.506 102.375 693.658 Q105.014 690.776 109.771 690.776 Q115.188 690.776 118.348 695.082 Q121.507 699.353 121.507 706.853 Q121.507 709.978 120.882 713.38 Q120.292 716.749 119.077 720.499 L112.479 720.499 Q114.32 716.957 115.257 713.519 Q116.16 710.082 116.16 706.714 Q116.16 702.2 114.632 699.769 Q113.07 697.339 110.257 697.339 Q107.653 697.339 106.264 699.11 Q104.875 700.846 103.591 706.783 L103.07 709.006 Q101.82 714.943 99.2504 717.582 Q96.6462 720.221 92.1324 720.221 Q86.6463 720.221 83.6602 716.332 Q80.6741 712.443 80.6741 705.29 Q80.6741 701.749 81.1949 698.624 Q81.7158 695.499 82.7574 692.86 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M99.4587 647.339 L102.584 647.339 L102.584 676.714 Q109.181 676.297 112.653 672.756 Q116.091 669.179 116.091 662.825 Q116.091 659.145 115.188 655.707 Q114.285 652.235 112.479 648.832 L118.521 648.832 Q119.979 652.27 120.743 655.881 Q121.507 659.492 121.507 663.207 Q121.507 672.513 116.091 677.964 Q110.674 683.381 101.438 683.381 Q91.8893 683.381 86.2991 678.242 Q80.6741 673.068 80.6741 664.318 Q80.6741 656.471 85.7435 651.922 Q90.7782 647.339 99.4587 647.339 M97.5837 653.728 Q92.3407 653.797 89.2157 656.679 Q86.0907 659.527 86.0907 664.249 Q86.0907 669.596 89.1116 672.825 Q92.1324 676.02 97.6185 676.506 L97.5837 653.728 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M83.1047 608.867 L89.0768 608.867 Q87.5838 611.575 86.8546 614.319 Q86.0907 617.027 86.0907 619.805 Q86.0907 626.02 90.0491 629.457 Q93.9726 632.895 101.091 632.895 Q108.209 632.895 112.167 629.457 Q116.091 626.02 116.091 619.805 Q116.091 617.027 115.361 614.319 Q114.598 611.575 113.104 608.867 L119.007 608.867 Q120.257 611.541 120.882 614.423 Q121.507 617.27 121.507 620.499 Q121.507 629.284 115.986 634.457 Q110.466 639.631 101.091 639.631 Q91.5768 639.631 86.1255 634.423 Q80.6741 629.18 80.6741 620.082 Q80.6741 617.131 81.2991 614.319 Q81.8894 611.506 83.1047 608.867 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M86.0907 582.687 Q86.0907 587.826 90.1185 590.812 Q94.1115 593.798 101.091 593.798 Q108.07 593.798 112.098 590.846 Q116.091 587.86 116.091 582.687 Q116.091 577.583 112.063 574.597 Q108.035 571.61 101.091 571.61 Q94.181 571.61 90.1532 574.597 Q86.0907 577.583 86.0907 582.687 M80.6741 582.687 Q80.6741 574.353 86.0907 569.597 Q91.5074 564.84 101.091 564.84 Q110.639 564.84 116.091 569.597 Q121.507 574.353 121.507 582.687 Q121.507 591.055 116.091 595.812 Q110.639 600.534 101.091 600.534 Q91.5074 600.534 86.0907 595.812 Q80.6741 591.055 80.6741 582.687 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M97.0282 521.923 L120.5 521.923 L120.5 528.312 L97.2365 528.312 Q91.7157 528.312 88.9727 530.465 Q86.2296 532.618 86.2296 536.923 Q86.2296 542.097 89.5282 545.083 Q92.8268 548.069 98.5212 548.069 L120.5 548.069 L120.5 554.492 L81.6116 554.492 L81.6116 548.069 L87.6532 548.069 Q84.1463 545.777 82.4102 542.687 Q80.6741 539.562 80.6741 535.5 Q80.6741 528.798 84.8408 525.361 Q88.9727 521.923 97.0282 521.923 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M87.5143 483.59 L66.4728 483.59 L66.4728 477.201 L120.5 477.201 L120.5 483.59 L114.667 483.59 Q118.139 485.604 119.841 488.694 Q121.507 491.75 121.507 496.055 Q121.507 503.104 115.882 507.548 Q110.257 511.958 101.091 511.958 Q91.924 511.958 86.2991 507.548 Q80.6741 503.104 80.6741 496.055 Q80.6741 491.75 82.3755 488.694 Q84.0421 485.604 87.5143 483.59 M101.091 505.361 Q108.139 505.361 112.167 502.479 Q116.16 499.562 116.16 494.493 Q116.16 489.423 112.167 486.507 Q108.139 483.59 101.091 483.59 Q94.0421 483.59 90.0491 486.507 Q86.0213 489.423 86.0213 494.493 Q86.0213 499.562 90.0491 502.479 Q94.0421 505.361 101.091 505.361 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M82.7574 439.25 L88.7991 439.25 Q87.4102 441.958 86.7157 444.875 Q86.0213 447.792 86.0213 450.917 Q86.0213 455.674 87.4796 458.069 Q88.938 460.43 91.8546 460.43 Q94.0768 460.43 95.3615 458.729 Q96.6115 457.028 97.7573 451.889 L98.2435 449.701 Q99.7018 442.896 102.375 440.049 Q105.014 437.167 109.771 437.167 Q115.188 437.167 118.348 441.472 Q121.507 445.743 121.507 453.243 Q121.507 456.368 120.882 459.771 Q120.292 463.139 119.077 466.889 L112.479 466.889 Q114.32 463.347 115.257 459.91 Q116.16 456.472 116.16 453.104 Q116.16 448.59 114.632 446.16 Q113.07 443.729 110.257 443.729 Q107.653 443.729 106.264 445.5 Q104.875 447.236 103.591 453.174 L103.07 455.396 Q101.82 461.333 99.2504 463.972 Q96.6462 466.611 92.1324 466.611 Q86.6463 466.611 83.6602 462.722 Q80.6741 458.833 80.6741 451.681 Q80.6741 448.139 81.1949 445.014 Q81.7158 441.889 82.7574 439.25 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M66.5422 428 L66.5422 422.445 Q74.7366 417.236 82.5838 414.667 Q90.431 412.063 98.174 412.063 Q105.952 412.063 113.834 414.667 Q121.716 417.236 129.875 422.445 L129.875 428 Q121.924 423.382 114.077 421.125 Q106.195 418.833 98.174 418.833 Q90.1532 418.833 82.3408 421.125 Q74.5283 423.382 66.5422 428 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><polyline clip-path="url(#clip412)" style="stroke:#009af9; stroke-linecap:round; stroke-linejoin:round; stroke-width:8; stroke-opacity:1; fill:none" points="
  426.307,1206.99 430.667,1206.99 435.028,1206.99 439.389,1206.99 443.75,1206.99 448.111,1206.99 452.472,1206.99 456.833,1206.99 461.193,1206.99 465.554,1206.99 
  469.915,1206.99 474.276,1206.99 478.637,1206.99 482.998,1206.99 487.359,1206.99 491.719,1206.99 496.08,1206.99 500.441,1206.99 504.802,1206.99 509.163,1206.99 
  513.524,1206.99 517.884,1206.99 522.245,1206.99 526.606,1206.99 530.967,1206.99 535.328,1206.99 539.689,1206.99 544.05,1206.99 548.41,1206.99 552.771,1206.99 
  557.132,1206.99 561.493,1206.99 565.854,1206.99 570.215,1206.99 574.576,1206.99 578.936,1206.99 583.297,1206.99 587.658,1206.99 592.019,1206.99 596.38,1206.99 
  600.741,1206.99 605.102,1206.99 609.462,1206.99 613.823,1206.99 618.184,1206.99 622.545,1206.99 626.906,1206.99 631.267,1206.99 635.628,1206.99 639.988,1206.99 
  644.349,1206.99 648.71,1206.99 653.071,1206.99 657.432,1206.99 661.793,1206.99 666.154,1206.99 670.514,1206.99 674.875,1206.99 679.236,1206.99 683.597,1206.99 
  687.958,1206.99 692.319,1206.99 696.679,1206.99 701.04,1206.99 705.401,1206.99 709.762,1206.99 714.123,1206.99 718.484,1206.99 722.845,1206.99 727.205,1206.99 
  731.566,1206.99 735.927,1206.99 740.288,1206.99 744.649,1206.99 749.01,1206.99 753.371,1206.99 757.731,1206.99 762.092,1206.99 766.453,1206.99 770.814,1206.99 
  775.175,1206.99 779.536,1206.99 783.897,1206.99 788.257,1206.99 792.618,1206.99 796.979,1206.99 801.34,1206.99 805.701,1206.99 810.062,1206.99 814.423,1206.99 
  818.783,1206.99 823.144,1206.99 827.505,1206.99 831.866,1206.99 836.227,1206.99 840.588,1206.99 844.949,1206.99 849.309,1206.99 853.67,1206.99 858.031,1206.99 
  862.392,1206.99 866.753,1206.99 871.114,1206.99 875.474,1206.99 879.835,1206.99 884.196,1206.99 888.557,1206.99 892.918,1206.99 897.279,1206.99 901.64,1206.99 
  906,1206.99 910.361,1206.99 914.722,1206.99 919.083,1206.99 923.444,1206.99 927.805,1206.99 932.166,1206.99 936.526,1206.99 940.887,1206.99 945.248,1206.99 
  949.609,1206.99 953.97,1206.99 958.331,1206.99 962.692,1206.99 967.052,1206.99 971.413,1206.99 975.774,1206.99 980.135,1206.99 984.496,1206.99 988.857,1206.99 
  993.218,1206.99 997.578,1206.99 1001.94,1206.99 1006.3,1206.99 1010.66,1206.99 1015.02,1206.99 1019.38,1206.99 1023.74,1206.99 1028.1,1206.99 1032.47,1206.99 
  1036.83,1206.99 1041.19,1206.99 1045.55,1206.99 1049.91,1206.99 1054.27,1206.99 1058.63,1206.99 1062.99,1206.99 1067.35,1206.99 1071.71,1206.99 1076.07,1206.99 
  1080.43,1206.99 1084.8,1206.99 1089.16,1206.99 1093.52,1206.99 1097.88,1206.99 1102.24,1206.99 1106.6,1206.99 1110.96,1206.99 1115.32,1206.99 1119.68,1206.99 
  1124.04,1206.99 1128.4,1206.99 1132.76,1206.99 1137.13,1206.99 1141.49,1206.99 1145.85,1206.99 1150.21,1206.99 1154.57,1206.99 1158.93,1206.99 1163.29,1206.99 
  1167.65,1206.99 1172.01,1206.99 1176.37,1206.99 1180.73,1206.99 1185.1,1206.99 1189.46,1206.99 1193.82,1206.99 1198.18,1206.99 1202.54,1206.99 1206.9,1206.99 
  1211.26,1206.99 1215.62,1206.99 1219.98,1206.99 1224.34,1206.99 1228.7,1206.99 1233.06,1206.99 1237.43,1206.99 1241.79,1206.99 1246.15,1206.99 1250.51,1206.99 
  1254.87,1206.99 1259.23,1206.99 1263.59,1206.99 1267.95,1206.99 1272.31,1206.99 1276.67,1206.99 1281.03,1206.99 1285.39,1206.99 1289.76,1206.99 1294.12,1206.99 
  1298.48,1206.99 1302.84,1206.99 1307.2,1206.99 1311.56,1206.99 1315.92,1206.99 1320.28,1206.99 1324.64,1206.99 1329,1206.99 1333.36,1206.99 1337.72,1206.99 
  1342.09,1206.99 1346.45,1206.99 1350.81,1206.99 1355.17,1206.99 1359.53,1206.99 1363.89,1206.99 1368.25,1206.99 1372.61,1206.99 1376.97,1206.99 1381.33,1206.99 
  1385.69,1206.99 1390.06,1206.99 1394.42,1206.99 1398.78,1206.99 1403.14,1206.99 1407.5,1206.99 1411.86,1206.99 1416.22,1206.99 1420.58,1206.99 1424.94,1206.99 
  1429.3,1206.99 1433.66,1206.99 1438.02,1206.99 1442.39,1206.99 1446.75,1206.99 1451.11,1206.99 1455.47,1206.99 1459.83,1206.99 1464.19,1206.99 1468.55,1206.99 
  1472.91,1206.99 1477.27,1206.99 1481.63,1206.99 1485.99,1206.99 1490.35,1206.99 1494.72,1206.99 1499.08,1206.99 1503.44,1206.99 1507.8,1206.99 1512.16,1206.99 
  1516.52,1206.99 1520.88,1206.99 1525.24,1206.99 1529.6,1206.99 1533.96,1206.99 1538.32,1206.99 1542.69,1206.99 1547.05,1206.99 1551.41,1206.99 1555.77,1206.99 
  1560.13,1206.99 1564.49,1206.99 1568.85,1206.99 1573.21,1206.99 1577.57,1206.99 1581.93,1206.99 1586.29,1206.99 1590.65,1206.99 1595.02,1206.99 1599.38,1206.99 
  1603.74,1206.99 1608.1,1206.99 1612.46,1206.99 1616.82,1206.99 1621.18,1206.99 1625.54,1206.99 1629.9,1206.99 1634.26,1206.99 1638.62,1206.99 1642.98,1206.99 
  1647.35,1206.99 1651.71,1206.99 1656.07,1206.99 1660.43,1206.99 1664.79,1206.99 1669.15,1206.99 1673.51,1206.99 1677.87,1206.99 1682.23,1206.99 1686.59,1206.99 
  1690.95,1206.99 1695.31,1206.99 1699.68,1206.99 1704.04,1206.99 1708.4,1206.99 1712.76,1206.99 1717.12,1206.99 1721.48,1206.99 1725.84,1206.99 1730.2,1206.99 
  1734.56,1206.99 1738.92,1206.99 1743.28,1206.99 1747.65,1206.99 1752.01,1206.99 1756.37,1206.99 1760.73,1206.99 1765.09,1206.99 1769.45,1206.99 1773.81,1206.99 
  1778.17,1206.99 1782.53,1206.99 1786.89,1206.99 1791.25,1206.99 1795.61,1206.99 1799.98,1206.99 1804.34,1206.99 1808.7,1206.99 1813.06,1206.99 1817.42,1206.99 
  1821.78,1206.99 1826.14,1206.99 1830.5,1206.99 1834.86,1206.99 1839.22,1206.99 1843.58,1206.99 1847.94,1206.99 1852.31,1206.99 1856.67,1206.99 1861.03,1206.99 
  1865.39,1206.99 1869.75,1206.99 1874.11,1206.99 1878.47,1206.99 1882.83,1206.99 1887.19,1206.99 1891.55,1206.99 1895.91,1206.99 1900.28,1206.99 1904.64,1206.99 
  1909,1206.99 1913.36,1206.99 1917.72,1206.99 1922.08,1206.99 1926.44,1206.99 1930.8,1206.99 1935.16,1206.99 1939.52,1206.99 1943.88,1206.99 1948.24,1206.99 
  1952.61,1206.99 1956.97,1206.99 1961.33,1206.99 1965.69,1206.99 1970.05,1206.99 1974.41,1206.99 1978.77,1206.99 1983.13,1206.99 1987.49,1206.99 1991.85,1206.99 
  1996.21,1206.99 2000.57,1206.99 2004.94,1206.99 2009.3,1206.99 2013.66,1206.99 2018.02,1206.99 2022.38,1206.99 2026.74,1206.99 2031.1,1206.99 2035.46,1206.99 
  2039.82,1206.99 2044.18,1206.99 2048.54,1206.99 2052.9,1206.99 2057.27,1206.99 2061.63,1206.99 2065.99,1206.99 2070.35,1206.99 2074.71,1206.99 2079.07,1206.99 
  2083.43,1206.99 2087.79,1206.99 2092.15,1206.99 2096.51,1206.99 2100.87,1206.99 2105.24,1206.99 2109.6,1206.99 2113.96,1206.99 2118.32,1206.99 2122.68,1206.99 
  2127.04,1206.99 2131.4,1206.99 2135.76,1206.99 2140.12,1206.99 2144.48,1206.99 2148.84,1206.99 2153.2,1206.99 2157.57,1206.99 2161.93,1206.99 2166.29,1206.99 
  2170.65,1206.99 2175.01,1206.99 2179.37,1206.99 2183.73,1206.99 2188.09,1206.99 2192.45,1206.99 2196.81,1206.99 2201.17,1206.99 2205.53,1206.99 2209.9,1206.99 
  2214.26,1206.99 2218.62,1206.99 2222.98,1206.99 2227.34,1206.99 2231.7,1206.99 2236.06,1206.99 2240.42,1206.99 2244.78,1206.99 2249.14,1206.99 2253.5,1206.99 
  2257.87,1206.99 2262.23,1206.99 2266.59,1206.99 2270.95,1206.99 2275.31,1206.99 2279.67,1206.99 2284.03,1206.99 2288.39,1206.99 2292.75,1206.99 2297.11,1206.99 
  2301.47,1206.99 2305.83,1206.99 2310.2,1206.99 2314.56,1206.99 2318.92,1206.99 2323.28,1206.99 2327.64,1206.99 2332,1206.99 2336.36,1206.99 2340.72,1206.99 
  2345.08,1206.99 2349.44,1206.99 2353.8,1206.99 2358.16,1206.99 2362.53,1206.99 2366.89,1206.99 2371.25,1206.99 2375.61,1206.99 2379.97,1206.99 2384.33,1206.99 
  2388.69,1206.99 2393.05,1206.99 2397.41,1206.99 2401.77,1206.99 2406.13,1206.99 2410.49,1206.99 2414.86,1206.99 2419.22,1206.99 2423.58,1206.99 2427.94,1206.99 
  2432.3,1206.99 2436.66,1206.99 2441.02,1206.99 2445.38,1206.99 2449.74,1206.99 2454.1,1206.99 2458.46,1206.99 2462.83,1206.99 2467.19,1206.99 2471.55,1206.99 
  2475.91,1206.99 2480.27,1206.99 2484.63,1206.99 2488.99,1206.99 2493.35,1206.99 2497.71,1206.99 2502.07,1206.99 2506.43,1206.99 2510.79,1206.99 2515.16,1206.99 
  2519.52,1206.99 2523.88,1206.99 2528.24,1206.99 2532.6,1206.99 2536.96,1206.99 2541.32,1206.99 2545.68,1206.99 2550.04,1206.99 2554.4,1206.99 2558.76,1206.99 
  2563.12,1206.99 2567.49,1206.99 2571.85,1206.99 2576.21,1206.99 2580.57,1206.99 2584.93,1206.99 2589.29,1206.99 2593.65,1206.99 2598.01,1206.99 2602.37,1206.99 
  2606.73,1206.99 2611.09,1206.99 2615.46,1206.99 2619.82,1206.99 2624.18,1206.99 2628.54,1206.99 2632.9,1206.99 2637.26,1206.99 2641.62,1206.99 2645.98,1206.99 
  2650.34,1206.99 2654.7,1206.99 2659.06,1206.99 2663.42,1206.99 2667.79,1206.99 2672.15,1206.99 2676.51,1206.99 2680.87,1206.99 2685.23,1206.99 2689.59,1206.99 
  2693.95,1206.99 2698.31,1206.99 2702.67,1206.99 2707.03,1206.99 2711.39,1206.99 2715.75,1206.99 2720.12,1206.99 2724.48,1206.99 2728.84,1206.99 2733.2,1206.99 
  2737.56,1206.99 2741.92,1206.99 2746.28,1206.99 2750.64,1206.99 2755,1206.99 2759.36,1206.99 2763.72,1206.99 2768.09,1206.99 2772.45,1206.99 2776.81,1206.99 
  2781.17,1206.99 2785.53,1206.99 2789.89,1206.99 2794.25,1206.99 2798.61,1206.99 2802.97,1206.99 2807.33,1206.99 2811.69,1206.99 2816.05,1206.99 2820.42,1206.99 
  2824.78,1206.99 2829.14,1206.99 2833.5,1206.99 2837.86,1206.99 2842.22,1206.99 2846.58,1206.99 2850.94,1206.99 2855.3,1206.99 2859.66,1206.99 2864.02,1206.99 
  2868.38,1206.99 2872.75,1206.99 2877.11,1206.99 2881.47,1206.99 2885.83,1206.99 2890.19,1206.99 2894.55,1206.99 2898.91,1206.99 2903.27,1206.99 2907.63,1206.99 
  2911.99,1206.99 2916.35,1206.99 2920.71,1206.99 2925.08,1206.99 2929.44,1206.99 2933.8,1206.99 2938.16,1206.99 2942.52,1206.99 2946.88,1206.99 2951.24,1206.99 
  2955.6,1206.99 2959.96,1206.99 2964.32,1206.99 2968.68,1206.99 2973.05,1206.99 2977.41,1206.99 2981.77,1206.99 2986.13,1206.99 2990.49,1206.99 2994.85,1206.99 
  2999.21,1206.99 3003.57,1206.99 3007.93,1206.99 3012.29,1206.99 3016.65,1206.99 3021.01,1206.99 3025.38,1206.99 3029.74,1206.99 3034.1,1206.99 3038.46,1206.99 
  3042.82,1206.99 3047.18,1206.99 3051.54,1206.99 3055.9,1206.99 3060.26,1206.99 3064.62,1206.99 3068.98,1206.99 3073.34,1206.99 
  "/>
<polyline clip-path="url(#clip412)" style="stroke:#e26f46; stroke-linecap:round; stroke-linejoin:round; stroke-width:8; stroke-opacity:1; fill:none" points="
  426.307,562.589 430.667,562.589 435.028,562.589 439.389,562.589 443.75,562.589 448.111,562.589 452.472,562.589 456.833,562.589 461.193,562.589 465.554,562.589 
  469.915,562.589 474.276,562.589 478.637,562.589 482.998,562.589 487.359,562.589 491.719,562.589 496.08,562.589 500.441,562.589 504.802,562.589 509.163,562.589 
  513.524,562.589 517.884,562.589 522.245,562.589 526.606,562.589 530.967,562.589 535.328,562.589 539.689,562.589 544.05,562.589 548.41,562.589 552.771,562.589 
  557.132,562.589 561.493,562.589 565.854,562.589 570.215,562.589 574.576,562.589 578.936,562.589 583.297,562.589 587.658,562.589 592.019,562.589 596.38,562.589 
  600.741,562.589 605.102,562.589 609.462,562.589 613.823,562.589 618.184,562.589 622.545,562.589 626.906,562.589 631.267,562.589 635.628,562.589 639.988,562.589 
  644.349,562.589 648.71,562.589 653.071,562.589 657.432,562.589 661.793,562.589 666.154,562.589 670.514,562.589 674.875,562.589 679.236,562.589 683.597,562.589 
  687.958,562.589 692.319,562.589 696.679,562.589 701.04,562.589 705.401,562.589 709.762,562.589 714.123,562.589 718.484,562.589 722.845,562.589 727.205,562.589 
  731.566,562.589 735.927,562.589 740.288,562.589 744.649,562.589 749.01,562.589 753.371,562.589 757.731,562.589 762.092,562.589 766.453,562.589 770.814,562.589 
  775.175,562.589 779.536,562.589 783.897,562.589 788.257,562.589 792.618,562.589 796.979,562.589 801.34,562.589 805.701,562.589 810.062,562.589 814.423,562.589 
  818.783,562.589 823.144,562.589 827.505,562.589 831.866,562.589 836.227,562.589 840.588,562.589 844.949,562.589 849.309,562.589 853.67,562.589 858.031,562.589 
  862.392,562.589 866.753,562.589 871.114,562.589 875.474,562.589 879.835,562.589 884.196,562.589 888.557,562.589 892.918,562.589 897.279,562.589 901.64,562.589 
  906,562.589 910.361,562.589 914.722,562.589 919.083,562.589 923.444,562.589 927.805,562.589 932.166,562.589 936.526,562.589 940.887,562.589 945.248,562.589 
  949.609,562.589 953.97,562.589 958.331,562.589 962.692,562.589 967.052,562.589 971.413,562.589 975.774,562.589 980.135,562.589 984.496,562.589 988.857,562.589 
  993.218,562.589 997.578,562.589 1001.94,562.589 1006.3,562.589 1010.66,562.589 1015.02,562.589 1019.38,562.589 1023.74,562.589 1028.1,562.589 1032.47,562.589 
  1036.83,562.589 1041.19,562.589 1045.55,562.589 1049.91,562.589 1054.27,562.589 1058.63,562.589 1062.99,562.589 1067.35,562.589 1071.71,562.589 1076.07,562.589 
  1080.43,562.589 1084.8,562.589 1089.16,562.589 1093.52,562.589 1097.88,562.589 1102.24,562.589 1106.6,562.589 1110.96,562.589 1115.32,562.589 1119.68,562.589 
  1124.04,562.589 1128.4,562.589 1132.76,562.589 1137.13,562.589 1141.49,562.589 1145.85,562.589 1150.21,562.589 1154.57,562.589 1158.93,562.589 1163.29,562.589 
  1167.65,562.589 1172.01,562.589 1176.37,562.589 1180.73,562.589 1185.1,562.589 1189.46,562.589 1193.82,562.589 1198.18,562.589 1202.54,562.589 1206.9,562.589 
  1211.26,562.589 1215.62,562.589 1219.98,562.589 1224.34,562.589 1228.7,562.589 1233.06,562.589 1237.43,562.589 1241.79,562.589 1246.15,562.589 1250.51,562.589 
  1254.87,562.589 1259.23,562.589 1263.59,562.589 1267.95,562.589 1272.31,562.589 1276.67,562.589 1281.03,562.589 1285.39,562.589 1289.76,562.589 1294.12,562.589 
  1298.48,562.589 1302.84,562.589 1307.2,562.589 1311.56,562.589 1315.92,562.589 1320.28,562.589 1324.64,562.589 1329,562.589 1333.36,562.589 1337.72,562.589 
  1342.09,562.589 1346.45,562.589 1350.81,562.589 1355.17,562.589 1359.53,562.589 1363.89,562.589 1368.25,562.589 1372.61,562.589 1376.97,562.589 1381.33,562.589 
  1385.69,562.589 1390.06,562.589 1394.42,562.589 1398.78,562.589 1403.14,562.589 1407.5,562.589 1411.86,562.589 1416.22,562.589 1420.58,562.589 1424.94,562.589 
  1429.3,562.589 1433.66,562.589 1438.02,562.589 1442.39,562.589 1446.75,562.589 1451.11,562.589 1455.47,562.589 1459.83,562.589 1464.19,562.589 1468.55,562.589 
  1472.91,562.589 1477.27,562.589 1481.63,562.589 1485.99,562.589 1490.35,562.589 1494.72,562.589 1499.08,562.589 1503.44,562.589 1507.8,562.589 1512.16,562.589 
  1516.52,562.589 1520.88,562.589 1525.24,562.589 1529.6,562.589 1533.96,562.589 1538.32,562.589 1542.69,562.589 1547.05,562.589 1551.41,562.589 1555.77,562.589 
  1560.13,562.589 1564.49,562.589 1568.85,562.589 1573.21,562.589 1577.57,562.589 1581.93,562.589 1586.29,562.589 1590.65,562.589 1595.02,562.589 1599.38,562.589 
  1603.74,562.589 1608.1,562.589 1612.46,562.589 1616.82,562.589 1621.18,562.589 1625.54,562.589 1629.9,562.589 1634.26,562.589 1638.62,562.589 1642.98,562.589 
  1647.35,562.589 1651.71,562.589 1656.07,562.589 1660.43,562.589 1664.79,562.589 1669.15,562.589 1673.51,562.589 1677.87,562.589 1682.23,562.589 1686.59,562.589 
  1690.95,562.589 1695.31,562.589 1699.68,562.589 1704.04,562.589 1708.4,562.589 1712.76,562.589 1717.12,562.589 1721.48,562.589 1725.84,562.589 1730.2,562.589 
  1734.56,562.589 1738.92,562.589 1743.28,562.589 1747.65,562.589 1752.01,562.589 1756.37,562.589 1760.73,562.589 1765.09,562.589 1769.45,562.589 1773.81,562.589 
  1778.17,562.589 1782.53,562.589 1786.89,562.589 1791.25,562.589 1795.61,562.589 1799.98,562.589 1804.34,562.589 1808.7,562.589 1813.06,562.589 1817.42,562.589 
  1821.78,562.589 1826.14,562.589 1830.5,562.589 1834.86,562.589 1839.22,562.589 1843.58,562.589 1847.94,562.589 1852.31,562.589 1856.67,562.589 1861.03,562.589 
  1865.39,562.589 1869.75,562.589 1874.11,562.589 1878.47,562.589 1882.83,562.589 1887.19,562.589 1891.55,562.589 1895.91,562.589 1900.28,562.589 1904.64,562.589 
  1909,562.589 1913.36,562.589 1917.72,562.589 1922.08,562.589 1926.44,562.589 1930.8,562.589 1935.16,562.589 1939.52,562.589 1943.88,562.589 1948.24,562.589 
  1952.61,562.589 1956.97,562.589 1961.33,562.589 1965.69,562.589 1970.05,562.589 1974.41,562.589 1978.77,562.589 1983.13,562.589 1987.49,562.589 1991.85,562.589 
  1996.21,562.589 2000.57,562.589 2004.94,562.589 2009.3,562.589 2013.66,562.589 2018.02,562.589 2022.38,562.589 2026.74,562.589 2031.1,562.589 2035.46,562.589 
  2039.82,562.589 2044.18,562.589 2048.54,562.589 2052.9,562.589 2057.27,562.589 2061.63,562.589 2065.99,562.589 2070.35,562.589 2074.71,562.589 2079.07,562.589 
  2083.43,562.589 2087.79,562.589 2092.15,562.589 2096.51,562.589 2100.87,562.589 2105.24,562.589 2109.6,562.589 2113.96,562.589 2118.32,562.589 2122.68,562.589 
  2127.04,562.589 2131.4,562.589 2135.76,562.589 2140.12,562.589 2144.48,562.589 2148.84,562.589 2153.2,562.589 2157.57,562.589 2161.93,562.589 2166.29,562.589 
  2170.65,562.589 2175.01,562.589 2179.37,562.589 2183.73,562.589 2188.09,562.589 2192.45,562.589 2196.81,562.589 2201.17,562.589 2205.53,562.589 2209.9,562.589 
  2214.26,562.589 2218.62,562.589 2222.98,562.589 2227.34,562.589 2231.7,562.589 2236.06,562.589 2240.42,562.589 2244.78,562.589 2249.14,562.589 2253.5,562.589 
  2257.87,562.589 2262.23,562.589 2266.59,562.589 2270.95,562.589 2275.31,562.589 2279.67,562.589 2284.03,562.589 2288.39,562.589 2292.75,562.589 2297.11,562.589 
  2301.47,562.589 2305.83,562.589 2310.2,562.589 2314.56,562.589 2318.92,562.589 2323.28,562.589 2327.64,562.589 2332,562.589 2336.36,562.589 2340.72,562.589 
  2345.08,562.589 2349.44,562.589 2353.8,562.589 2358.16,562.589 2362.53,562.589 2366.89,562.589 2371.25,562.589 2375.61,562.589 2379.97,562.589 2384.33,562.589 
  2388.69,562.589 2393.05,562.589 2397.41,562.589 2401.77,562.589 2406.13,562.589 2410.49,562.589 2414.86,562.589 2419.22,562.589 2423.58,562.589 2427.94,562.589 
  2432.3,562.589 2436.66,562.589 2441.02,562.589 2445.38,562.589 2449.74,562.589 2454.1,562.589 2458.46,562.589 2462.83,562.589 2467.19,562.589 2471.55,562.589 
  2475.91,562.589 2480.27,562.589 2484.63,562.589 2488.99,562.589 2493.35,562.589 2497.71,562.589 2502.07,562.589 2506.43,562.589 2510.79,562.589 2515.16,562.589 
  2519.52,562.589 2523.88,562.589 2528.24,562.589 2532.6,562.589 2536.96,562.589 2541.32,562.589 2545.68,562.589 2550.04,562.589 2554.4,562.589 2558.76,562.589 
  2563.12,562.589 2567.49,562.589 2571.85,562.589 2576.21,562.589 2580.57,562.589 2584.93,562.589 2589.29,562.589 2593.65,562.589 2598.01,562.589 2602.37,562.589 
  2606.73,562.589 2611.09,562.589 2615.46,562.589 2619.82,562.589 2624.18,562.589 2628.54,562.589 2632.9,562.589 2637.26,562.589 2641.62,562.589 2645.98,562.589 
  2650.34,562.589 2654.7,562.589 2659.06,562.589 2663.42,562.589 2667.79,562.589 2672.15,562.589 2676.51,562.589 2680.87,562.589 2685.23,562.589 2689.59,562.589 
  2693.95,562.589 2698.31,562.589 2702.67,562.589 2707.03,562.589 2711.39,562.589 2715.75,562.589 2720.12,562.589 2724.48,562.589 2728.84,562.589 2733.2,562.589 
  2737.56,562.589 2741.92,562.589 2746.28,562.589 2750.64,562.589 2755,562.589 2759.36,562.589 2763.72,562.589 2768.09,562.589 2772.45,562.589 2776.81,562.589 
  2781.17,562.589 2785.53,562.589 2789.89,562.589 2794.25,562.589 2798.61,562.589 2802.97,562.589 2807.33,562.589 2811.69,562.589 2816.05,562.589 2820.42,562.589 
  2824.78,562.589 2829.14,562.589 2833.5,562.589 2837.86,562.589 2842.22,562.589 2846.58,562.589 2850.94,562.589 2855.3,562.589 2859.66,562.589 2864.02,562.589 
  2868.38,562.589 2872.75,562.589 2877.11,562.589 2881.47,562.589 2885.83,562.589 2890.19,562.589 2894.55,562.589 2898.91,562.589 2903.27,562.589 2907.63,562.589 
  2911.99,562.589 2916.35,562.589 2920.71,562.589 2925.08,562.589 2929.44,562.589 2933.8,562.589 2938.16,562.589 2942.52,562.589 2946.88,562.589 2951.24,562.589 
  2955.6,562.589 2959.96,562.589 2964.32,562.589 2968.68,562.589 2973.05,562.589 2977.41,562.589 2981.77,562.589 2986.13,562.589 2990.49,562.589 2994.85,562.589 
  2999.21,562.589 3003.57,562.589 3007.93,562.589 3012.29,562.589 3016.65,562.589 3021.01,562.589 3025.38,562.589 3029.74,562.589 3034.1,562.589 3038.46,562.589 
  3042.82,562.589 3047.18,562.589 3051.54,562.589 3055.9,562.589 3060.26,562.589 3064.62,562.589 3068.98,562.589 3073.34,562.589 
  "/>
<polyline clip-path="url(#clip412)" style="stroke:#3da44d; stroke-linecap:round; stroke-linejoin:round; stroke-width:8; stroke-opacity:1; fill:none" points="
  426.307,562.573 430.667,562.561 435.028,562.556 439.389,562.559 443.75,562.57 448.111,562.586 452.472,562.602 456.833,562.615 461.193,562.622 465.554,562.619 
  469.915,562.609 474.276,562.593 478.637,562.576 482.998,562.562 487.359,562.556 491.719,562.558 496.08,562.568 500.441,562.583 504.802,562.6 509.163,562.614 
  513.524,562.621 517.884,562.62 522.245,562.611 526.606,562.596 530.967,562.579 535.328,562.564 539.689,562.556 544.05,562.557 548.41,562.566 552.771,562.58 
  557.132,562.597 561.493,562.611 565.854,562.62 570.215,562.621 574.576,562.613 578.936,562.599 583.297,562.582 587.658,562.566 592.019,562.557 596.38,562.556 
  600.741,562.564 605.102,562.577 609.462,562.594 613.823,562.609 618.184,562.619 622.545,562.622 626.906,562.615 631.267,562.602 635.628,562.585 639.988,562.569 
  644.349,562.558 648.71,562.556 653.071,562.562 657.432,562.575 661.793,562.591 666.154,562.607 670.514,562.618 674.875,562.622 679.236,562.617 683.597,562.605 
  687.958,562.588 692.319,562.571 696.679,562.56 701.04,562.556 705.401,562.56 709.762,562.572 714.123,562.588 718.484,562.604 722.845,562.617 727.205,562.622 
  731.566,562.618 735.927,562.607 740.288,562.591 744.649,562.574 749.01,562.561 753.371,562.556 757.731,562.559 762.092,562.569 766.453,562.585 770.814,562.602 
  775.175,562.615 779.536,562.621 783.897,562.62 788.257,562.61 792.618,562.594 796.979,562.577 801.34,562.563 805.701,562.556 810.062,562.558 814.423,562.567 
  818.783,562.582 823.144,562.599 827.505,562.613 831.866,562.621 836.227,562.621 840.588,562.612 844.949,562.597 849.309,562.58 853.67,562.565 858.031,562.557 
  862.392,562.557 866.753,562.565 871.114,562.579 875.474,562.596 879.835,562.611 884.196,562.62 888.557,562.621 892.918,562.614 897.279,562.6 901.64,562.583 
  906,562.567 910.361,562.557 914.722,562.556 919.083,562.563 923.444,562.577 927.805,562.593 932.166,562.608 936.526,562.619 940.887,562.622 945.248,562.616 
  949.609,562.603 953.97,562.586 958.331,562.57 962.692,562.559 967.052,562.556 971.413,562.561 975.774,562.574 980.135,562.59 984.496,562.606 988.857,562.618 
  993.218,562.622 997.578,562.617 1001.94,562.605 1006.3,562.589 1010.66,562.572 1015.02,562.56 1019.38,562.556 1023.74,562.56 1028.1,562.571 1032.47,562.587 
  1036.83,562.603 1041.19,562.616 1045.55,562.622 1049.91,562.619 1054.27,562.608 1058.63,562.592 1062.99,562.575 1067.35,562.562 1071.71,562.556 1076.07,562.558 
  1080.43,562.569 1084.8,562.584 1089.16,562.601 1093.52,562.614 1097.88,562.621 1102.24,562.62 1106.6,562.61 1110.96,562.595 1115.32,562.578 1119.68,562.564 
  1124.04,562.556 1128.4,562.557 1132.76,562.566 1137.13,562.581 1141.49,562.598 1145.85,562.612 1150.21,562.621 1154.57,562.621 1158.93,562.613 1163.29,562.598 
  1167.65,562.581 1172.01,562.566 1176.37,562.557 1180.73,562.556 1185.1,562.564 1189.46,562.578 1193.82,562.595 1198.18,562.61 1202.54,562.62 1206.9,562.621 
  1211.26,562.615 1215.62,562.601 1219.98,562.584 1224.34,562.568 1228.7,562.558 1233.06,562.556 1237.43,562.562 1241.79,562.576 1246.15,562.592 1250.51,562.608 
  1254.87,562.619 1259.23,562.622 1263.59,562.616 1267.95,562.603 1272.31,562.587 1276.67,562.57 1281.03,562.559 1285.39,562.556 1289.76,562.561 1294.12,562.573 
  1298.48,562.589 1302.84,562.605 1307.2,562.617 1311.56,562.622 1315.92,562.618 1320.28,562.606 1324.64,562.59 1329,562.573 1333.36,562.56 1337.72,562.556 
  1342.09,562.559 1346.45,562.57 1350.81,562.586 1355.17,562.603 1359.53,562.616 1363.89,562.622 1368.25,562.619 1372.61,562.609 1376.97,562.593 1381.33,562.576 
  1385.69,562.562 1390.06,562.556 1394.42,562.558 1398.78,562.568 1403.14,562.583 1407.5,562.6 1411.86,562.614 1416.22,562.621 1420.58,562.62 1424.94,562.611 
  1429.3,562.596 1433.66,562.579 1438.02,562.564 1442.39,562.556 1446.75,562.557 1451.11,562.566 1455.47,562.58 1459.83,562.597 1464.19,562.612 1468.55,562.62 
  1472.91,562.621 1477.27,562.613 1481.63,562.599 1485.99,562.581 1490.35,562.566 1494.72,562.557 1499.08,562.556 1503.44,562.564 1507.8,562.578 1512.16,562.594 
  1516.52,562.609 1520.88,562.619 1525.24,562.622 1529.6,562.615 1533.96,562.602 1538.32,562.584 1542.69,562.569 1547.05,562.558 1551.41,562.556 1555.77,562.562 
  1560.13,562.575 1564.49,562.591 1568.85,562.607 1573.21,562.618 1577.57,562.622 1581.93,562.617 1586.29,562.604 1590.65,562.588 1595.02,562.571 1599.38,562.559 
  1603.74,562.556 1608.1,562.56 1612.46,562.572 1616.82,562.588 1621.18,562.605 1625.54,562.617 1629.9,562.622 1634.26,562.618 1638.62,562.607 1642.98,562.591 
  1647.35,562.574 1651.71,562.561 1656.07,562.556 1660.43,562.559 1664.79,562.57 1669.15,562.585 1673.51,562.602 1677.87,562.615 1682.23,562.622 1686.59,562.62 
  1690.95,562.609 1695.31,562.594 1699.68,562.577 1704.04,562.563 1708.4,562.556 1712.76,562.558 1717.12,562.567 1721.48,562.582 1725.84,562.599 1730.2,562.613 
  1734.56,562.621 1738.92,562.621 1743.28,562.612 1747.65,562.597 1752.01,562.579 1756.37,562.565 1760.73,562.556 1765.09,562.557 1769.45,562.565 1773.81,562.58 
  1778.17,562.596 1782.53,562.611 1786.89,562.62 1791.25,562.621 1795.61,562.614 1799.98,562.6 1804.34,562.582 1808.7,562.567 1813.06,562.557 1817.42,562.556 
  1821.78,562.563 1826.14,562.577 1830.5,562.593 1834.86,562.609 1839.22,562.619 1843.58,562.622 1847.94,562.616 1852.31,562.602 1856.67,562.585 1861.03,562.569 
  1865.39,562.558 1869.75,562.556 1874.11,562.561 1878.47,562.574 1882.83,562.59 1887.19,562.606 1891.55,562.618 1895.91,562.622 1900.28,562.617 1904.64,562.605 
  1909,562.588 1913.36,562.572 1917.72,562.56 1922.08,562.556 1926.44,562.56 1930.8,562.572 1935.16,562.587 1939.52,562.604 1943.88,562.616 1948.24,562.622 
  1952.61,562.619 1956.97,562.608 1961.33,562.591 1965.69,562.575 1970.05,562.561 1974.41,562.556 1978.77,562.558 1983.13,562.569 1987.49,562.585 1991.85,562.601 
  1996.21,562.615 2000.57,562.621 2004.94,562.62 2009.3,562.61 2013.66,562.594 2018.02,562.577 2022.38,562.563 2026.74,562.556 2031.1,562.557 2035.46,562.567 
  2039.82,562.582 2044.18,562.598 2048.54,562.613 2052.9,562.621 2057.27,562.621 2061.63,562.612 2065.99,562.597 2070.35,562.58 2074.71,562.565 2079.07,562.557 
  2083.43,562.557 2087.79,562.565 2092.15,562.579 2096.51,562.595 2100.87,562.61 2105.24,562.62 2109.6,562.621 2113.96,562.614 2118.32,562.6 2122.68,562.583 
  2127.04,562.568 2131.4,562.558 2135.76,562.556 2140.12,562.563 2144.48,562.576 2148.84,562.592 2153.2,562.608 2157.57,562.619 2161.93,562.622 2166.29,562.616 
  2170.65,562.603 2175.01,562.586 2179.37,562.57 2183.73,562.559 2188.09,562.556 2192.45,562.561 2196.81,562.573 2201.17,562.59 2205.53,562.606 2209.9,562.617 
  2214.26,562.622 2218.62,562.618 2222.98,562.606 2227.34,562.589 2231.7,562.573 2236.06,562.56 2240.42,562.556 2244.78,562.559 2249.14,562.571 2253.5,562.587 
  2257.87,562.603 2262.23,562.616 2266.59,562.622 2270.95,562.619 2275.31,562.608 2279.67,562.592 2284.03,562.575 2288.39,562.562 2292.75,562.556 2297.11,562.558 
  2301.47,562.568 2305.83,562.584 2310.2,562.6 2314.56,562.614 2318.92,562.621 2323.28,562.62 2327.64,562.611 2332,562.595 2336.36,562.578 2340.72,562.564 
  2345.08,562.556 2349.44,562.557 2353.8,562.566 2358.16,562.581 2362.53,562.597 2366.89,562.612 2371.25,562.621 2375.61,562.621 2379.97,562.613 2384.33,562.598 
  2388.69,562.581 2393.05,562.566 2397.41,562.557 2401.77,562.556 2406.13,562.564 2410.49,562.578 2414.86,562.595 2419.22,562.61 2423.58,562.62 2427.94,562.622 
  2432.3,562.615 2436.66,562.601 2441.02,562.584 2445.38,562.568 2449.74,562.558 2454.1,562.556 2458.46,562.562 2462.83,562.575 2467.19,562.592 2471.55,562.607 
  2475.91,562.618 2480.27,562.622 2484.63,562.617 2488.99,562.604 2493.35,562.587 2497.71,562.571 2502.07,562.559 2506.43,562.556 2510.79,562.56 2515.16,562.573 
  2519.52,562.589 2523.88,562.605 2528.24,562.617 2532.6,562.622 2536.96,562.618 2541.32,562.607 2545.68,562.59 2550.04,562.573 2554.4,562.561 2558.76,562.556 
  2563.12,562.559 2567.49,562.57 2571.85,562.586 2576.21,562.602 2580.57,562.615 2584.93,562.622 2589.29,562.619 2593.65,562.609 2598.01,562.593 2602.37,562.576 
  2606.73,562.563 2611.09,562.556 2615.46,562.558 2619.82,562.568 2624.18,562.583 2628.54,562.599 2632.9,562.613 2637.26,562.621 2641.62,562.62 2645.98,562.611 
  2650.34,562.596 2654.7,562.579 2659.06,562.564 2663.42,562.556 2667.79,562.557 2672.15,562.565 2676.51,562.58 2680.87,562.597 2685.23,562.611 2689.59,562.62 
  2693.95,562.621 2698.31,562.614 2702.67,562.599 2707.03,562.582 2711.39,562.567 2715.75,562.557 2720.12,562.556 2724.48,562.563 2728.84,562.577 2733.2,562.594 
  2737.56,562.609 2741.92,562.619 2746.28,562.622 2750.64,562.615 2755,562.602 2759.36,562.585 2763.72,562.569 2768.09,562.558 2772.45,562.556 2776.81,562.562 
  2781.17,562.574 2785.53,562.591 2789.89,562.607 2794.25,562.618 2798.61,562.622 2802.97,562.617 2807.33,562.605 2811.69,562.588 2816.05,562.572 2820.42,562.56 
  2824.78,562.556 2829.14,562.56 2833.5,562.572 2837.86,562.588 2842.22,562.604 2846.58,562.616 2850.94,562.622 2855.3,562.619 2859.66,562.607 2864.02,562.591 
  2868.38,562.574 2872.75,562.561 2877.11,562.556 2881.47,562.559 2885.83,562.569 2890.19,562.585 2894.55,562.601 2898.91,562.615 2903.27,562.621 2907.63,562.62 
  2911.99,562.61 2916.35,562.594 2920.71,562.577 2925.08,562.563 2929.44,562.556 2933.8,562.557 2938.16,562.567 2942.52,562.582 2946.88,562.599 2951.24,562.613 
  2955.6,562.621 2959.96,562.621 2964.32,562.612 2968.68,562.597 2973.05,562.58 2977.41,562.565 2981.77,562.557 2986.13,562.557 2990.49,562.565 2994.85,562.579 
  2999.21,562.596 3003.57,562.611 3007.93,562.62 3012.29,562.621 3016.65,562.614 3021.01,562.6 3025.38,562.583 3029.74,562.567 3034.1,562.558 3038.46,562.556 
  3042.82,562.563 3047.18,562.576 3051.54,562.593 3055.9,562.608 3060.26,562.619 3064.62,562.622 3068.98,562.616 3073.34,562.603 
  "/>
<polyline clip-path="url(#clip412)" style="stroke:#c271d2; stroke-linecap:round; stroke-linejoin:round; stroke-width:8; stroke-opacity:1; fill:none" points="
  426.307,591.167 430.667,590.349 435.028,589.54 439.389,588.738 443.75,587.945 448.111,587.156 452.472,586.368 456.833,585.576 461.193,584.777 465.554,583.97 
  469.915,583.155 474.276,582.335 478.637,581.513 482.998,580.695 487.359,579.884 491.719,579.081 496.08,578.286 500.441,577.497 504.802,576.709 509.163,575.918 
  513.524,575.121 517.884,574.315 522.245,573.501 526.606,572.682 530.967,571.86 535.328,571.04 539.689,570.228 544.05,569.424 548.41,568.628 552.771,567.837 
  557.132,567.049 561.493,566.259 565.854,565.464 570.215,564.66 574.576,563.847 578.936,563.028 583.297,562.206 587.658,561.386 592.019,560.572 596.38,559.767 
  600.741,558.969 605.102,558.178 609.462,557.39 613.823,556.601 618.184,555.806 622.545,555.004 626.906,554.193 631.267,553.375 635.628,552.553 639.988,551.732 
  644.349,550.917 648.71,550.11 653.071,549.311 657.432,548.519 661.793,547.731 666.154,546.942 670.514,546.149 674.875,545.348 679.236,544.538 683.597,543.721 
  687.958,542.9 692.319,542.079 696.679,541.262 701.04,540.453 705.401,539.653 709.762,538.86 714.123,538.072 718.484,537.283 722.845,536.491 727.205,535.691 
  731.566,534.883 735.927,534.067 740.288,533.246 744.649,532.425 749.01,531.607 753.371,530.797 757.731,529.996 762.092,529.202 766.453,528.413 770.814,527.624 
  775.175,526.833 779.536,526.035 783.897,525.228 788.257,524.414 792.618,523.593 796.979,522.771 801.34,521.953 805.701,521.141 810.062,520.338 814.423,519.543 
  818.783,518.753 823.144,517.965 827.505,517.175 831.866,516.378 836.227,515.573 840.588,514.76 844.949,513.94 849.309,513.118 853.67,512.299 858.031,511.485 
  862.392,510.681 866.753,509.885 871.114,509.094 875.474,508.306 879.835,507.516 884.196,506.721 888.557,505.917 892.918,505.105 897.279,504.287 901.64,503.465 
  906,502.644 910.361,501.83 914.722,501.024 919.083,500.226 923.444,499.435 927.805,498.647 932.166,497.858 936.526,497.063 940.887,496.262 945.248,495.451 
  949.609,494.633 953.97,493.811 958.331,492.991 962.692,492.175 967.052,491.367 971.413,490.568 975.774,489.776 980.135,488.988 984.496,488.199 988.857,487.406 
  993.218,486.605 997.578,485.796 1001.94,484.979 1006.3,484.158 1010.66,483.337 1015.02,482.52 1019.38,481.711 1023.74,480.91 1028.1,480.117 1032.47,479.328 
  1036.83,478.54 1041.19,477.748 1045.55,476.949 1049.91,476.141 1054.27,475.326 1058.63,474.505 1062.99,473.683 1067.35,472.865 1071.71,472.055 1076.07,471.253 
  1080.43,470.458 1084.8,469.669 1089.16,468.881 1093.52,468.09 1097.88,467.292 1102.24,466.486 1106.6,465.672 1110.96,464.852 1115.32,464.03 1119.68,463.211 
  1124.04,462.399 1128.4,461.595 1132.76,460.8 1137.13,460.01 1141.49,459.222 1145.85,458.432 1150.21,457.635 1154.57,456.831 1158.93,456.018 1163.29,455.198 
  1167.65,454.376 1172.01,453.557 1176.37,452.743 1180.73,451.938 1185.1,451.141 1189.46,450.351 1193.82,449.563 1198.18,448.773 1202.54,447.978 1206.9,447.175 
  1211.26,446.363 1215.62,445.545 1219.98,444.723 1224.34,443.903 1228.7,443.088 1233.06,442.281 1237.43,441.483 1241.79,440.692 1246.15,439.904 1250.51,439.114 
  1254.87,438.321 1259.23,437.519 1263.59,436.709 1267.95,435.891 1272.31,435.07 1276.67,434.249 1281.03,433.433 1285.39,432.625 1289.76,431.825 1294.12,431.033 
  1298.48,430.244 1302.84,429.456 1307.2,428.663 1311.56,427.863 1315.92,427.054 1320.28,426.238 1324.64,425.417 1329,424.595 1333.36,423.778 1337.72,422.968 
  1342.09,422.167 1346.45,421.374 1350.81,420.585 1355.17,419.797 1359.53,419.005 1363.89,418.206 1368.25,417.399 1372.61,416.584 1376.97,415.763 1381.33,414.942 
  1385.69,414.123 1390.06,413.312 1394.42,412.51 1398.78,411.715 1403.14,410.926 1407.5,410.138 1411.86,409.347 1416.22,408.55 1420.58,407.744 1424.94,406.93 
  1429.3,406.11 1433.66,405.288 1438.02,404.469 1442.39,403.657 1446.75,402.853 1451.11,402.057 1455.47,401.267 1459.83,400.479 1464.19,399.689 1468.55,398.893 
  1472.91,398.088 1477.27,397.276 1481.63,396.457 1485.99,395.635 1490.35,394.815 1494.72,394.001 1499.08,393.196 1503.44,392.398 1507.8,391.608 1512.16,390.819 
  1516.52,390.03 1520.88,389.235 1525.24,388.433 1529.6,387.622 1533.96,386.803 1538.32,385.982 1542.69,385.161 1547.05,384.346 1551.41,383.539 1555.77,382.74 
  1560.13,381.949 1564.49,381.16 1568.85,380.371 1573.21,379.578 1577.57,378.777 1581.93,377.967 1586.29,377.15 1590.65,376.328 1595.02,375.507 1599.38,374.691 
  1603.74,373.882 1608.1,373.082 1612.46,372.29 1616.82,371.501 1621.18,370.712 1625.54,369.92 1629.9,369.12 1634.26,368.312 1638.62,367.496 1642.98,366.675 
  1647.35,365.854 1651.71,365.036 1656.07,364.226 1660.43,363.425 1664.79,362.631 1669.15,361.842 1673.51,361.054 1677.87,360.262 1682.23,359.464 1686.59,358.657 
  1690.95,357.842 1695.31,357.022 1699.68,356.2 1704.04,355.382 1708.4,354.57 1712.76,353.767 1717.12,352.972 1721.48,352.183 1725.84,351.394 1730.2,350.604 
  1734.56,349.807 1738.92,349.002 1743.28,348.188 1747.65,347.368 1752.01,346.547 1756.37,345.727 1760.73,344.914 1765.09,344.11 1769.45,343.314 1773.81,342.523 
  1778.17,341.735 1782.53,340.945 1786.89,340.15 1791.25,339.346 1795.61,338.534 1799.98,337.715 1804.34,336.893 1808.7,336.073 1813.06,335.259 1817.42,334.453 
  1821.78,333.655 1826.14,332.864 1830.5,332.076 1834.86,331.287 1839.22,330.492 1843.58,329.69 1847.94,328.88 1852.31,328.062 1856.67,327.24 1861.03,326.419 
  1865.39,325.604 1869.75,324.796 1874.11,323.997 1878.47,323.205 1882.83,322.417 1887.19,321.628 1891.55,320.835 1895.91,320.034 1900.28,319.225 1904.64,318.408 
  1909,317.587 1913.36,316.765 1917.72,315.949 1922.08,315.14 1926.44,314.339 1930.8,313.546 1935.16,312.758 1939.52,311.969 1943.88,311.177 1948.24,310.378 
  1952.61,309.57 1956.97,308.754 1961.33,307.933 1965.69,307.112 1970.05,306.294 1974.41,305.484 1978.77,304.682 1983.13,303.888 1987.49,303.098 1991.85,302.31 
  1996.21,301.519 2000.57,300.721 2004.94,299.915 2009.3,299.1 2013.66,298.28 2018.02,297.458 2022.38,296.64 2026.74,295.828 2031.1,295.024 2035.46,294.229 
  2039.82,293.439 2044.18,292.651 2048.54,291.861 2052.9,291.064 2057.27,290.26 2061.63,289.446 2065.99,288.627 2070.35,287.805 2074.71,286.985 2079.07,286.172 
  2083.43,285.367 2087.79,284.571 2092.15,283.78 2096.51,282.992 2100.87,282.202 2105.24,281.407 2109.6,280.604 2113.96,279.792 2118.32,278.973 2122.68,278.152 
  2127.04,277.331 2131.4,276.517 2135.76,275.71 2140.12,274.912 2144.48,274.121 2148.84,273.333 2153.2,272.544 2157.57,271.75 2161.93,270.948 2166.29,270.138 
  2170.65,269.32 2175.01,268.498 2179.37,267.677 2183.73,266.862 2188.09,266.054 2192.45,265.254 2196.81,264.462 2201.17,263.674 2205.53,262.885 2209.9,262.092 
  2214.26,261.292 2218.62,260.483 2222.98,259.666 2227.34,258.845 2231.7,258.024 2236.06,257.207 2240.42,256.397 2244.78,255.596 2249.14,254.803 2253.5,254.014 
  2257.87,253.226 2262.23,252.434 2266.59,251.635 2270.95,250.828 2275.31,250.013 2279.67,249.192 2284.03,248.37 2288.39,247.552 2292.75,246.741 2297.11,245.939 
  2301.47,245.144 2305.83,244.355 2310.2,243.567 2314.56,242.776 2318.92,241.979 2323.28,241.173 2327.64,240.359 2332,239.539 2336.36,238.717 2340.72,237.898 
  2345.08,237.085 2349.44,236.282 2353.8,235.486 2358.16,234.696 2362.53,233.908 2366.89,233.118 2371.25,232.322 2375.61,231.517 2379.97,230.705 2384.33,229.885 
  2388.69,229.063 2393.05,228.244 2397.41,227.43 2401.77,226.624 2406.13,225.828 2410.49,225.037 2414.86,224.249 2419.22,223.459 2423.58,222.664 2427.94,221.862 
  2432.3,221.05 2436.66,220.232 2441.02,219.41 2445.38,218.59 2449.74,217.775 2454.1,216.968 2458.46,216.169 2462.83,215.378 2467.19,214.589 2471.55,213.8 
  2475.91,213.007 2480.27,212.206 2484.63,211.396 2488.99,210.578 2493.35,209.757 2497.71,208.936 2502.07,208.12 2506.43,207.311 2510.79,206.511 2515.16,205.719 
  2519.52,204.93 2523.88,204.142 2528.24,203.349 2532.6,202.549 2536.96,201.741 2541.32,200.925 2545.68,200.104 2550.04,199.282 2554.4,198.465 2558.76,197.655 
  2563.12,196.854 2567.49,196.06 2571.85,195.271 2576.21,194.483 2580.57,193.691 2584.93,192.893 2589.29,192.086 2593.65,191.271 2598.01,190.45 2602.37,189.629 
  2606.73,188.81 2611.09,187.999 2615.46,187.196 2619.82,186.401 2624.18,185.612 2628.54,184.824 2632.9,184.033 2637.26,183.236 2641.62,182.431 2645.98,181.617 
  2650.34,180.797 2654.7,179.975 2659.06,179.156 2663.42,178.343 2667.79,177.539 2672.15,176.743 2676.51,175.953 2680.87,175.165 2685.23,174.375 2689.59,173.579 
  2693.95,172.775 2698.31,171.963 2702.67,171.144 2707.03,170.322 2711.39,169.502 2715.75,168.688 2720.12,167.882 2724.48,167.084 2728.84,166.293 2733.2,165.505 
  2737.56,164.716 2741.92,163.922 2746.28,163.119 2750.64,162.308 2755,161.49 2759.36,160.669 2763.72,159.848 2768.09,159.032 2772.45,158.225 2776.81,157.426 
  2781.17,156.634 2785.53,155.846 2789.89,155.057 2794.25,154.264 2798.61,153.463 2802.97,152.654 2807.33,151.837 2811.69,151.015 2816.05,150.194 2820.42,149.377 
  2824.78,148.569 2829.14,147.768 2833.5,146.976 2837.86,146.187 2842.22,145.398 2846.58,144.606 2850.94,143.807 2855.3,142.999 2859.66,142.183 2864.02,141.362 
  2868.38,140.54 2872.75,139.723 2877.11,138.912 2881.47,138.111 2885.83,137.317 2890.19,136.528 2894.55,135.739 2898.91,134.948 2903.27,134.15 2907.63,133.344 
  2911.99,132.529 2916.35,131.709 2920.71,130.887 2925.08,130.068 2929.44,129.257 2933.8,128.453 2938.16,127.658 2942.52,126.868 2946.88,126.08 2951.24,125.29 
  2955.6,124.493 2959.96,123.688 2964.32,122.875 2968.68,122.055 2973.05,121.234 2977.41,120.414 2981.77,119.601 2986.13,118.796 2990.49,118 2994.85,117.209 
  2999.21,116.421 3003.57,115.631 3007.93,114.836 3012.29,114.033 3016.65,113.221 3021.01,112.402 3025.38,111.58 3029.74,110.76 3034.1,109.945 3038.46,109.139 
  3042.82,108.341 3047.18,107.55 3051.54,106.762 3055.9,105.973 3060.26,105.179 3064.62,104.377 3068.98,103.566 3073.34,102.749 
  "/>
<polyline clip-path="url(#clip412)" style="stroke:#ac8d18; stroke-linecap:round; stroke-linejoin:round; stroke-width:8; stroke-opacity:1; fill:none" points="
  426.307,563.874 430.667,563.838 435.028,563.802 439.389,563.765 443.75,563.729 448.111,563.693 452.472,563.657 456.833,563.621 461.193,563.585 465.554,563.548 
  469.915,563.512 474.276,563.476 478.637,563.44 482.998,563.404 487.359,563.368 491.719,563.331 496.08,563.295 500.441,563.259 504.802,563.223 509.163,563.187 
  513.524,563.151 517.884,563.114 522.245,563.078 526.606,563.042 530.967,563.006 535.328,562.97 539.689,562.934 544.05,562.897 548.41,562.861 552.771,562.825 
  557.132,562.789 561.493,562.753 565.854,562.716 570.215,562.68 574.576,562.644 578.936,562.608 583.297,562.572 587.658,562.536 592.019,562.499 596.38,562.463 
  600.741,562.427 605.102,562.391 609.462,562.355 613.823,562.319 618.184,562.282 622.545,562.246 626.906,562.21 631.267,562.174 635.628,562.138 639.988,562.102 
  644.349,562.065 648.71,562.029 653.071,561.993 657.432,561.957 661.793,561.921 666.154,561.885 670.514,561.848 674.875,561.812 679.236,561.776 683.597,561.74 
  687.958,561.704 692.319,561.668 696.679,561.631 701.04,561.595 705.401,561.559 709.762,561.523 714.123,561.487 718.484,561.451 722.845,561.414 727.205,561.378 
  731.566,561.342 735.927,561.306 740.288,561.27 744.649,561.234 749.01,561.197 753.371,561.161 757.731,561.125 762.092,561.089 766.453,561.053 770.814,561.017 
  775.175,560.98 779.536,560.944 783.897,560.908 788.257,560.872 792.618,560.836 796.979,560.8 801.34,560.763 805.701,560.727 810.062,560.691 814.423,560.655 
  818.783,560.619 823.144,560.582 827.505,560.546 831.866,560.51 836.227,560.474 840.588,560.438 844.949,560.402 849.309,560.365 853.67,560.329 858.031,560.293 
  862.392,560.257 866.753,560.221 871.114,560.185 875.474,560.148 879.835,560.112 884.196,560.076 888.557,560.04 892.918,560.004 897.279,559.968 901.64,559.931 
  906,559.895 910.361,559.859 914.722,559.823 919.083,559.787 923.444,559.751 927.805,559.714 932.166,559.678 936.526,559.642 940.887,559.606 945.248,559.57 
  949.609,559.534 953.97,559.497 958.331,559.461 962.692,559.425 967.052,559.389 971.413,559.353 975.774,559.317 980.135,559.28 984.496,559.244 988.857,559.208 
  993.218,559.172 997.578,559.136 1001.94,559.1 1006.3,559.063 1010.66,559.027 1015.02,558.991 1019.38,558.955 1023.74,558.919 1028.1,558.883 1032.47,558.846 
  1036.83,558.81 1041.19,558.774 1045.55,558.738 1049.91,558.702 1054.27,558.666 1058.63,558.629 1062.99,558.593 1067.35,558.557 1071.71,558.521 1076.07,558.485 
  1080.43,558.449 1084.8,558.412 1089.16,558.376 1093.52,558.34 1097.88,558.304 1102.24,558.268 1106.6,558.231 1110.96,558.195 1115.32,558.159 1119.68,558.123 
  1124.04,558.087 1128.4,558.051 1132.76,558.014 1137.13,557.978 1141.49,557.942 1145.85,557.906 1150.21,557.87 1154.57,557.834 1158.93,557.797 1163.29,557.761 
  1167.65,557.725 1172.01,557.689 1176.37,557.653 1180.73,557.617 1185.1,557.58 1189.46,557.544 1193.82,557.508 1198.18,557.472 1202.54,557.436 1206.9,557.4 
  1211.26,557.363 1215.62,557.327 1219.98,557.291 1224.34,557.255 1228.7,557.219 1233.06,557.183 1237.43,557.146 1241.79,557.11 1246.15,557.074 1250.51,557.038 
  1254.87,557.002 1259.23,556.966 1263.59,556.929 1267.95,556.893 1272.31,556.857 1276.67,556.821 1281.03,556.785 1285.39,556.749 1289.76,556.712 1294.12,556.676 
  1298.48,556.64 1302.84,556.604 1307.2,556.568 1311.56,556.532 1315.92,556.495 1320.28,556.459 1324.64,556.423 1329,556.387 1333.36,556.351 1337.72,556.315 
  1342.09,556.278 1346.45,556.242 1350.81,556.206 1355.17,556.17 1359.53,556.134 1363.89,556.098 1368.25,556.061 1372.61,556.025 1376.97,555.989 1381.33,555.953 
  1385.69,555.917 1390.06,555.88 1394.42,555.844 1398.78,555.808 1403.14,555.772 1407.5,555.736 1411.86,555.7 1416.22,555.663 1420.58,555.627 1424.94,555.591 
  1429.3,555.555 1433.66,555.519 1438.02,555.483 1442.39,555.446 1446.75,555.41 1451.11,555.374 1455.47,555.338 1459.83,555.302 1464.19,555.266 1468.55,555.229 
  1472.91,555.193 1477.27,555.157 1481.63,555.121 1485.99,555.085 1490.35,555.049 1494.72,555.012 1499.08,554.976 1503.44,554.94 1507.8,554.904 1512.16,554.868 
  1516.52,554.832 1520.88,554.795 1525.24,554.759 1529.6,554.723 1533.96,554.687 1538.32,554.651 1542.69,554.615 1547.05,554.578 1551.41,554.542 1555.77,554.506 
  1560.13,554.47 1564.49,554.434 1568.85,554.398 1573.21,554.361 1577.57,554.325 1581.93,554.289 1586.29,554.253 1590.65,554.217 1595.02,554.181 1599.38,554.144 
  1603.74,554.108 1608.1,554.072 1612.46,554.036 1616.82,554 1621.18,553.964 1625.54,553.927 1629.9,553.891 1634.26,553.855 1638.62,553.819 1642.98,553.783 
  1647.35,553.746 1651.71,553.71 1656.07,553.674 1660.43,553.638 1664.79,553.602 1669.15,553.566 1673.51,553.529 1677.87,553.493 1682.23,553.457 1686.59,553.421 
  1690.95,553.385 1695.31,553.349 1699.68,553.312 1704.04,553.276 1708.4,553.24 1712.76,553.204 1717.12,553.168 1721.48,553.132 1725.84,553.095 1730.2,553.059 
  1734.56,553.023 1738.92,552.987 1743.28,552.951 1747.65,552.915 1752.01,552.878 1756.37,552.842 1760.73,552.806 1765.09,552.77 1769.45,552.734 1773.81,552.698 
  1778.17,552.661 1782.53,552.625 1786.89,552.589 1791.25,552.553 1795.61,552.517 1799.98,552.481 1804.34,552.444 1808.7,552.408 1813.06,552.372 1817.42,552.336 
  1821.78,552.3 1826.14,552.264 1830.5,552.227 1834.86,552.191 1839.22,552.155 1843.58,552.119 1847.94,552.083 1852.31,552.047 1856.67,552.01 1861.03,551.974 
  1865.39,551.938 1869.75,551.902 1874.11,551.866 1878.47,551.83 1882.83,551.793 1887.19,551.757 1891.55,551.721 1895.91,551.685 1900.28,551.649 1904.64,551.613 
  1909,551.576 1913.36,551.54 1917.72,551.504 1922.08,551.468 1926.44,551.432 1930.8,551.395 1935.16,551.359 1939.52,551.323 1943.88,551.287 1948.24,551.251 
  1952.61,551.215 1956.97,551.178 1961.33,551.142 1965.69,551.106 1970.05,551.07 1974.41,551.034 1978.77,550.998 1983.13,550.961 1987.49,550.925 1991.85,550.889 
  1996.21,550.853 2000.57,550.817 2004.94,550.781 2009.3,550.744 2013.66,550.708 2018.02,550.672 2022.38,550.636 2026.74,550.6 2031.1,550.564 2035.46,550.527 
  2039.82,550.491 2044.18,550.455 2048.54,550.419 2052.9,550.383 2057.27,550.347 2061.63,550.31 2065.99,550.274 2070.35,550.238 2074.71,550.202 2079.07,550.166 
  2083.43,550.13 2087.79,550.093 2092.15,550.057 2096.51,550.021 2100.87,549.985 2105.24,549.949 2109.6,549.913 2113.96,549.876 2118.32,549.84 2122.68,549.804 
  2127.04,549.768 2131.4,549.732 2135.76,549.696 2140.12,549.659 2144.48,549.623 2148.84,549.587 2153.2,549.551 2157.57,549.515 2161.93,549.479 2166.29,549.442 
  2170.65,549.406 2175.01,549.37 2179.37,549.334 2183.73,549.298 2188.09,549.261 2192.45,549.225 2196.81,549.189 2201.17,549.153 2205.53,549.117 2209.9,549.081 
  2214.26,549.044 2218.62,549.008 2222.98,548.972 2227.34,548.936 2231.7,548.9 2236.06,548.864 2240.42,548.827 2244.78,548.791 2249.14,548.755 2253.5,548.719 
  2257.87,548.683 2262.23,548.647 2266.59,548.61 2270.95,548.574 2275.31,548.538 2279.67,548.502 2284.03,548.466 2288.39,548.43 2292.75,548.393 2297.11,548.357 
  2301.47,548.321 2305.83,548.285 2310.2,548.249 2314.56,548.213 2318.92,548.176 2323.28,548.14 2327.64,548.104 2332,548.068 2336.36,548.032 2340.72,547.996 
  2345.08,547.959 2349.44,547.923 2353.8,547.887 2358.16,547.851 2362.53,547.815 2366.89,547.779 2371.25,547.742 2375.61,547.706 2379.97,547.67 2384.33,547.634 
  2388.69,547.598 2393.05,547.562 2397.41,547.525 2401.77,547.489 2406.13,547.453 2410.49,547.417 2414.86,547.381 2419.22,547.345 2423.58,547.308 2427.94,547.272 
  2432.3,547.236 2436.66,547.2 2441.02,547.164 2445.38,547.128 2449.74,547.091 2454.1,547.055 2458.46,547.019 2462.83,546.983 2467.19,546.947 2471.55,546.91 
  2475.91,546.874 2480.27,546.838 2484.63,546.802 2488.99,546.766 2493.35,546.73 2497.71,546.693 2502.07,546.657 2506.43,546.621 2510.79,546.585 2515.16,546.549 
  2519.52,546.513 2523.88,546.476 2528.24,546.44 2532.6,546.404 2536.96,546.368 2541.32,546.332 2545.68,546.296 2550.04,546.259 2554.4,546.223 2558.76,546.187 
  2563.12,546.151 2567.49,546.115 2571.85,546.079 2576.21,546.042 2580.57,546.006 2584.93,545.97 2589.29,545.934 2593.65,545.898 2598.01,545.862 2602.37,545.825 
  2606.73,545.789 2611.09,545.753 2615.46,545.717 2619.82,545.681 2624.18,545.645 2628.54,545.608 2632.9,545.572 2637.26,545.536 2641.62,545.5 2645.98,545.464 
  2650.34,545.428 2654.7,545.391 2659.06,545.355 2663.42,545.319 2667.79,545.283 2672.15,545.247 2676.51,545.211 2680.87,545.174 2685.23,545.138 2689.59,545.102 
  2693.95,545.066 2698.31,545.03 2702.67,544.994 2707.03,544.957 2711.39,544.921 2715.75,544.885 2720.12,544.849 2724.48,544.813 2728.84,544.776 2733.2,544.74 
  2737.56,544.704 2741.92,544.668 2746.28,544.632 2750.64,544.596 2755,544.559 2759.36,544.523 2763.72,544.487 2768.09,544.451 2772.45,544.415 2776.81,544.379 
  2781.17,544.342 2785.53,544.306 2789.89,544.27 2794.25,544.234 2798.61,544.198 2802.97,544.162 2807.33,544.125 2811.69,544.089 2816.05,544.053 2820.42,544.017 
  2824.78,543.981 2829.14,543.945 2833.5,543.908 2837.86,543.872 2842.22,543.836 2846.58,543.8 2850.94,543.764 2855.3,543.728 2859.66,543.691 2864.02,543.655 
  2868.38,543.619 2872.75,543.583 2877.11,543.547 2881.47,543.511 2885.83,543.474 2890.19,543.438 2894.55,543.402 2898.91,543.366 2903.27,543.33 2907.63,543.294 
  2911.99,543.257 2916.35,543.221 2920.71,543.185 2925.08,543.149 2929.44,543.113 2933.8,543.077 2938.16,543.04 2942.52,543.004 2946.88,542.968 2951.24,542.932 
  2955.6,542.896 2959.96,542.86 2964.32,542.823 2968.68,542.787 2973.05,542.751 2977.41,542.715 2981.77,542.679 2986.13,542.643 2990.49,542.606 2994.85,542.57 
  2999.21,542.534 3003.57,542.498 3007.93,542.462 3012.29,542.425 3016.65,542.389 3021.01,542.353 3025.38,542.317 3029.74,542.281 3034.1,542.245 3038.46,542.208 
  3042.82,542.172 3047.18,542.136 3051.54,542.1 3055.9,542.064 3060.26,542.028 3064.62,541.991 3068.98,541.955 3073.34,541.919 
  "/>
<polyline clip-path="url(#clip412)" style="stroke:#00a9ad; stroke-linecap:round; stroke-linejoin:round; stroke-width:8; stroke-opacity:1; fill:none" points="
  426.307,1467.28 430.667,1467.28 435.028,1467.28 439.389,1467.28 443.75,1467.28 448.111,1467.28 452.472,1467.28 456.833,1467.28 461.193,1467.28 465.554,1467.28 
  469.915,1467.28 474.276,1467.28 478.637,1487.31 482.998,1487.31 487.359,1487.31 491.719,1487.31 496.08,1487.31 500.441,1487.31 504.802,1487.31 509.163,1487.31 
  513.524,1487.31 517.884,1487.31 522.245,1487.31 526.606,1487.31 530.967,1507.33 535.328,1507.33 539.689,1507.33 544.05,1507.33 548.41,1507.33 552.771,1507.33 
  557.132,1507.33 561.493,1507.33 565.854,1507.33 570.215,1507.33 574.576,1507.33 578.936,1507.33 583.297,1527.35 587.658,1527.35 592.019,1527.35 596.38,1527.35 
  600.741,1527.35 605.102,1527.35 609.462,1527.35 613.823,1527.35 618.184,1527.35 622.545,1527.35 626.906,1527.35 631.267,1527.35 635.628,1547.37 639.988,1547.37 
  644.349,1547.37 648.71,1547.37 653.071,1547.37 657.432,1547.37 661.793,1547.37 666.154,1547.37 670.514,1547.37 674.875,1547.37 679.236,1547.37 683.597,1547.37 
  687.958,1567.39 692.319,1567.39 696.679,1567.39 701.04,1567.39 705.401,1567.39 709.762,1567.39 714.123,1567.39 718.484,1567.39 722.845,1567.39 727.205,1567.39 
  731.566,1567.39 735.927,1567.39 740.288,1567.39 744.649,1587.42 749.01,1587.42 753.371,1587.42 757.731,1587.42 762.092,1587.42 766.453,1587.42 770.814,1587.42 
  775.175,1587.42 779.536,1587.42 783.897,1587.42 788.257,1587.42 792.618,1587.42 796.979,1587.42 801.34,1587.42 805.701,1587.42 810.062,1587.42 814.423,1587.42 
  818.783,1587.42 823.144,1607.44 827.505,1607.44 831.866,1607.44 836.227,1607.44 840.588,1607.44 844.949,1607.44 849.309,1607.44 853.67,1607.44 858.031,1607.44 
  862.392,1607.44 866.753,1607.44 871.114,1607.44 875.474,1627.46 879.835,1627.46 884.196,1627.46 888.557,1627.46 892.918,1627.46 897.279,1627.46 901.64,1627.46 
  906,1627.46 910.361,1627.46 914.722,1627.46 919.083,1627.46 923.444,1627.46 927.805,1647.48 932.166,1647.48 936.526,1647.48 940.887,1647.48 945.248,1647.48 
  949.609,1647.48 953.97,1647.48 958.331,1647.48 962.692,1647.48 967.052,1647.48 971.413,1647.48 975.774,1647.48 980.135,1647.48 984.496,1647.48 988.857,1647.48 
  993.218,1647.48 997.578,1647.48 1001.94,1647.48 1006.3,1647.48 1010.66,1647.48 1015.02,1647.48 1019.38,1647.48 1023.74,1647.48 1028.1,1647.48 1032.47,1667.51 
  1036.83,1667.51 1041.19,1667.51 1045.55,1667.51 1049.91,1667.51 1054.27,1667.51 1058.63,1667.51 1062.99,1667.51 1067.35,1667.51 1071.71,1667.51 1076.07,1667.51 
  1080.43,1667.51 1084.8,1667.51 1089.16,1667.51 1093.52,1667.51 1097.88,1667.51 1102.24,1667.51 1106.6,1667.51 1110.96,1667.51 1115.32,1667.51 1119.68,1667.51 
  1124.04,1667.51 1128.4,1667.51 1132.76,1667.51 1137.13,1667.51 1141.49,1667.51 1145.85,1667.51 1150.21,1667.51 1154.57,1667.51 1158.93,1667.51 1163.29,1667.51 
  1167.65,1687.53 1172.01,1687.53 1176.37,1687.53 1180.73,1687.53 1185.1,1687.53 1189.46,1687.53 1193.82,1687.53 1198.18,1687.53 1202.54,1687.53 1206.9,1687.53 
  1211.26,1687.53 1215.62,1687.53 1219.98,1687.53 1224.34,1687.53 1228.7,1687.53 1233.06,1687.53 1237.43,1687.53 1241.79,1687.53 1246.15,1687.53 1250.51,1687.53 
  1254.87,1687.53 1259.23,1687.53 1263.59,1687.53 1267.95,1687.53 1272.31,1707.55 1276.67,1707.55 1281.03,1707.55 1285.39,1707.55 1289.76,1707.55 1294.12,1707.55 
  1298.48,1707.55 1302.84,1707.55 1307.2,1707.55 1311.56,1707.55 1315.92,1707.55 1320.28,1707.55 1324.64,1727.57 1329,1727.57 1333.36,1727.57 1337.72,1727.57 
  1342.09,1727.57 1346.45,1727.57 1350.81,1727.57 1355.17,1727.57 1359.53,1727.57 1363.89,1727.57 1368.25,1727.57 1372.61,1727.57 1376.97,1727.57 1381.33,1727.57 
  1385.69,1727.57 1390.06,1727.57 1394.42,1727.57 1398.78,1727.57 1403.14,1727.57 1407.5,1747.6 1411.86,1747.6 1416.22,1747.6 1420.58,1747.6 1424.94,1747.6 
  1429.3,1747.6 1433.66,1747.6 1438.02,1747.6 1442.39,1747.6 1446.75,1747.6 1451.11,1747.6 1455.47,1747.6 1459.83,1767.62 1464.19,1767.62 1468.55,1767.62 
  1472.91,1767.62 1477.27,1767.62 1481.63,1767.62 1485.99,1767.62 1490.35,1767.62 1494.72,1767.62 1499.08,1767.62 1503.44,1767.62 1507.8,1767.62 1512.16,1787.64 
  1516.52,1787.64 1520.88,1787.64 1525.24,1787.64 1529.6,1787.64 1533.96,1787.64 1538.32,1787.64 1542.69,1787.64 1547.05,1787.64 1551.41,1787.64 1555.77,1787.64 
  1560.13,1787.64 1564.49,1787.64 1568.85,1787.64 1573.21,1787.64 1577.57,1787.64 1581.93,1787.64 1586.29,1787.64 1590.65,1807.66 1595.02,1807.66 1599.38,1807.66 
  1603.74,1807.66 1608.1,1807.66 1612.46,1807.66 1616.82,1807.66 1621.18,1807.66 1625.54,1807.66 1629.9,1807.66 1634.26,1807.66 1638.62,1807.66 1642.98,1807.66 
  1647.35,1807.66 1651.71,1807.66 1656.07,1807.66 1660.43,1807.66 1664.79,1807.66 1669.15,1807.66 1673.51,1827.69 1677.87,1827.69 1682.23,1827.69 1686.59,1827.69 
  1690.95,1827.69 1695.31,1827.69 1699.68,1827.69 1704.04,1827.69 1708.4,1827.69 1712.76,1827.69 1717.12,1827.69 1721.48,1827.69 1725.84,1827.69 1730.2,1827.69 
  1734.56,1827.69 1738.92,1827.69 1743.28,1827.69 1747.65,1827.69 1752.01,1847.71 1756.37,1847.71 1760.73,1847.71 1765.09,1847.71 1769.45,1847.71 1773.81,1847.71 
  1778.17,1847.71 1782.53,1847.71 1786.89,1847.71 1791.25,1847.71 1795.61,1847.71 1799.98,1847.71 1804.34,1847.71 1808.7,1847.71 1813.06,1847.71 1817.42,1847.71 
  1821.78,1847.71 1826.14,1847.71 1830.5,1847.71 1834.86,1847.71 1839.22,1847.71 1843.58,1847.71 1847.94,1847.71 1852.31,1847.71 1856.67,1847.71 1861.03,1847.71 
  1865.39,1847.71 1869.75,1847.71 1874.11,1847.71 1878.47,1847.71 1882.83,1847.71 1887.19,1847.71 1891.55,1847.71 1895.91,1847.71 1900.28,1847.71 1904.64,1847.71 
  1909,1847.71 1913.36,1847.71 1917.72,1847.71 1922.08,1847.71 1926.44,1847.71 1930.8,1847.71 1935.16,1847.71 1939.52,1847.71 1943.88,1847.71 1948.24,1847.71 
  1952.61,1847.71 1956.97,1847.71 1961.33,1847.71 1965.69,1847.71 1970.05,1847.71 1974.41,1847.71 1978.77,1847.71 1983.13,1847.71 1987.49,1847.71 1991.85,1847.71 
  1996.21,1847.71 2000.57,1847.71 2004.94,1847.71 2009.3,1847.71 2013.66,1847.71 2018.02,1847.71 2022.38,1847.71 2026.74,1847.71 2031.1,1847.71 2035.46,1847.71 
  2039.82,1847.71 2044.18,1847.71 2048.54,1847.71 2052.9,1847.71 2057.27,1847.71 2061.63,1847.71 2065.99,1847.71 2070.35,1847.71 2074.71,1847.71 2079.07,1847.71 
  2083.43,1847.71 2087.79,1847.71 2092.15,1847.71 2096.51,1847.71 2100.87,1847.71 2105.24,1847.71 2109.6,1847.71 2113.96,1847.71 2118.32,1847.71 2122.68,1867.73 
  2127.04,1867.73 2131.4,1867.73 2135.76,1867.73 2140.12,1867.73 2144.48,1867.73 2148.84,1867.73 2153.2,1867.73 2157.57,1867.73 2161.93,1867.73 2166.29,1867.73 
  2170.65,1867.73 2175.01,1867.73 2179.37,1867.73 2183.73,1867.73 2188.09,1867.73 2192.45,1867.73 2196.81,1867.73 2201.17,1867.73 2205.53,1867.73 2209.9,1867.73 
  2214.26,1867.73 2218.62,1867.73 2222.98,1867.73 2227.34,1867.73 2231.7,1867.73 2236.06,1867.73 2240.42,1867.73 2244.78,1867.73 2249.14,1867.73 2253.5,1867.73 
  2257.87,1867.73 2262.23,1867.73 2266.59,1867.73 2270.95,1867.73 2275.31,1867.73 2279.67,1867.73 2284.03,1887.75 2288.39,1887.75 2292.75,1887.75 2297.11,1887.75 
  2301.47,1887.75 2305.83,1887.75 2310.2,1887.75 2314.56,1887.75 2318.92,1887.75 2323.28,1887.75 2327.64,1887.75 2332,1887.75 2336.36,1887.75 2340.72,1887.75 
  2345.08,1887.75 2349.44,1887.75 2353.8,1887.75 2358.16,1887.75 2362.53,1887.75 2366.89,1887.75 2371.25,1887.75 2375.61,1887.75 2379.97,1887.75 2384.33,1887.75 
  2388.69,1887.75 2393.05,1887.75 2397.41,1887.75 2401.77,1887.75 2406.13,1887.75 2410.49,1887.75 2414.86,1887.75 2419.22,1887.75 2423.58,1887.75 2427.94,1887.75 
  2432.3,1887.75 2436.66,1887.75 2441.02,1887.75 2445.38,1887.75 2449.74,1887.75 2454.1,1887.75 2458.46,1887.75 2462.83,1887.75 2467.19,1907.78 2471.55,1907.78 
  2475.91,1907.78 2480.27,1907.78 2484.63,1907.78 2488.99,1907.78 2493.35,1907.78 2497.71,1907.78 2502.07,1907.78 2506.43,1907.78 2510.79,1907.78 2515.16,1907.78 
  2519.52,1907.78 2523.88,1907.78 2528.24,1907.78 2532.6,1907.78 2536.96,1907.78 2541.32,1907.78 2545.68,1907.78 2550.04,1907.78 2554.4,1907.78 2558.76,1907.78 
  2563.12,1907.78 2567.49,1907.78 2571.85,1907.78 2576.21,1907.78 2580.57,1907.78 2584.93,1907.78 2589.29,1907.78 2593.65,1907.78 2598.01,1907.78 2602.37,1907.78 
  2606.73,1907.78 2611.09,1907.78 2615.46,1907.78 2619.82,1907.78 2624.18,1907.78 2628.54,1927.8 2632.9,1927.8 2637.26,1927.8 2641.62,1927.8 2645.98,1927.8 
  2650.34,1927.8 2654.7,1927.8 2659.06,1927.8 2663.42,1927.8 2667.79,1927.8 2672.15,1927.8 2676.51,1927.8 2680.87,1927.8 2685.23,1927.8 2689.59,1927.8 
  2693.95,1927.8 2698.31,1927.8 2702.67,1927.8 2707.03,1947.82 2711.39,1947.82 2715.75,1947.82 2720.12,1947.82 2724.48,1947.82 2728.84,1947.82 2733.2,1947.82 
  2737.56,1947.82 2741.92,1947.82 2746.28,1947.82 2750.64,1947.82 2755,1947.82 2759.36,1947.82 2763.72,1947.82 2768.09,1947.82 2772.45,1947.82 2776.81,1947.82 
  2781.17,1947.82 2785.53,1947.82 2789.89,1947.82 2794.25,1947.82 2798.61,1947.82 2802.97,1947.82 2807.33,1947.82 2811.69,1947.82 2816.05,1947.82 2820.42,1947.82 
  2824.78,1947.82 2829.14,1947.82 2833.5,1947.82 2837.86,1947.82 2842.22,1947.82 2846.58,1947.82 2850.94,1947.82 2855.3,1947.82 2859.66,1947.82 2864.02,1947.82 
  2868.38,1947.82 2872.75,1947.82 2877.11,1947.82 2881.47,1947.82 2885.83,1947.82 2890.19,1947.82 2894.55,1947.82 2898.91,1947.82 2903.27,1947.82 2907.63,1947.82 
  2911.99,1947.82 2916.35,1947.82 2920.71,1947.82 2925.08,1947.82 2929.44,1947.82 2933.8,1947.82 2938.16,1947.82 2942.52,1947.82 2946.88,1947.82 2951.24,1947.82 
  2955.6,1947.82 2959.96,1947.82 2964.32,1947.82 2968.68,1947.82 2973.05,1947.82 2977.41,1947.82 2981.77,1947.82 2986.13,1947.82 2990.49,1947.82 2994.85,1947.82 
  2999.21,1947.82 3003.57,1947.82 3007.93,1947.82 3012.29,1947.82 3016.65,1947.82 3021.01,1947.82 3025.38,1947.82 3029.74,1947.82 3034.1,1947.82 3038.46,1947.82 
  3042.82,1947.82 3047.18,1947.82 3051.54,1947.82 3055.9,1947.82 3060.26,1947.82 3064.62,1947.82 3068.98,1947.82 3073.34,1947.82 
  "/>
<polyline clip-path="url(#clip412)" style="stroke:#ed5d92; stroke-linecap:round; stroke-linejoin:round; stroke-width:8; stroke-opacity:1; fill:none" points="
  426.307,1454.83 430.667,1456.49 435.028,1458.36 439.389,1460.34 443.75,1462.14 448.111,1463.54 452.472,1464.7 456.833,1466.03 461.193,1467.56 465.554,1469.32 
  469.915,1471.18 474.276,1472.86 478.637,1474.48 482.998,1476.23 487.359,1478.04 491.719,1479.9 496.08,1481.6 500.441,1483.01 504.802,1484.17 509.163,1485.42 
  513.524,1486.87 517.884,1488.63 522.245,1490.49 526.606,1492.18 530.967,1493.88 535.328,1495.67 539.689,1497.54 544.05,1499.5 548.41,1501.44 552.771,1503.08 
  557.132,1504.45 561.493,1505.84 565.854,1507.42 570.215,1509.43 574.576,1511.37 578.936,1513.16 583.297,1514.85 587.658,1516.49 592.019,1518.25 596.38,1520.19 
  600.741,1522.12 605.102,1523.66 609.462,1524.84 613.823,1526 618.184,1527.43 622.545,1529.24 626.906,1531.19 631.267,1533.03 635.628,1534.96 639.988,1536.88 
  644.349,1538.99 648.71,1541.05 653.071,1542.96 657.432,1544.72 661.793,1545.99 666.154,1547.13 670.514,1548.56 674.875,1550.21 679.236,1552.03 683.597,1553.9 
  687.958,1555.69 692.319,1557.5 696.679,1559.2 701.04,1561.04 705.401,1562.91 709.762,1564.54 714.123,1565.88 718.484,1567.03 722.845,1568.28 727.205,1569.71 
  731.566,1571.38 735.927,1572.92 740.288,1574.45 744.649,1575.96 749.01,1577.41 753.371,1578.93 757.731,1580.53 762.092,1581.99 766.453,1583.16 770.814,1584.14 
  775.175,1585.17 779.536,1586.55 783.897,1588.08 788.257,1589.58 792.618,1591.02 796.979,1592.45 801.34,1593.78 805.701,1595.4 810.062,1597 814.423,1598.47 
  818.783,1599.74 823.144,1600.54 827.505,1601.49 831.866,1602.51 836.227,1603.89 840.588,1605.29 844.949,1606.61 849.309,1607.91 853.67,1609.12 858.031,1610.51 
  862.392,1612.03 866.753,1613.52 871.114,1614.8 875.474,1615.74 879.835,1616.56 884.196,1617.79 888.557,1619.19 892.918,1620.58 897.279,1622.07 901.64,1623.58 
  906,1625.35 910.361,1627.12 914.722,1628.9 919.083,1630.46 923.444,1631.81 927.805,1632.79 932.166,1633.72 936.526,1634.78 940.887,1635.96 945.248,1637.32 
  949.609,1638.64 953.97,1639.82 958.331,1640.73 962.692,1641.77 967.052,1642.95 971.413,1644.11 975.774,1645.02 980.135,1645.64 984.496,1646.1 988.857,1646.83 
  993.218,1647.74 997.578,1648.81 1001.94,1649.83 1006.3,1650.73 1010.66,1651.65 1015.02,1652.52 1019.38,1653.68 1023.74,1654.83 1028.1,1655.81 1032.47,1656.54 
  1036.83,1656.88 1041.19,1657.34 1045.55,1658.13 1049.91,1659.23 1054.27,1660.28 1058.63,1661.18 1062.99,1661.98 1067.35,1662.91 1071.71,1663.73 1076.07,1664.7 
  1080.43,1665.56 1084.8,1666 1089.16,1666.43 1093.52,1666.85 1097.88,1667.48 1102.24,1668.42 1106.6,1669.27 1110.96,1670.04 1115.32,1670.84 1119.68,1671.7 
  1124.04,1672.77 1128.4,1673.76 1132.76,1674.67 1137.13,1675.38 1141.49,1675.77 1145.85,1676.1 1150.21,1676.8 1154.57,1677.78 1158.93,1678.82 1163.29,1679.79 
  1167.65,1680.72 1172.01,1681.58 1176.37,1682.72 1180.73,1683.85 1185.1,1684.83 1189.46,1685.57 1193.82,1685.85 1198.18,1686.21 1202.54,1686.66 1206.9,1687.44 
  1211.26,1688.39 1215.62,1689.37 1219.98,1690.1 1224.34,1690.98 1228.7,1691.86 1233.06,1692.79 1237.43,1693.86 1241.79,1694.82 1246.15,1695.4 1250.51,1696.01 
  1254.87,1696.69 1259.23,1697.55 1263.59,1698.78 1267.95,1700.12 1272.31,1701.22 1276.67,1702.31 1281.03,1703.64 1285.39,1705.08 1289.76,1706.39 1294.12,1707.53 
  1298.48,1708.44 1302.84,1709.14 1307.2,1710.12 1311.56,1711.32 1315.92,1712.66 1320.28,1714 1324.64,1715.23 1329,1716.55 1333.36,1717.95 1337.72,1719.35 
  1342.09,1720.74 1346.45,1722 1350.81,1723.04 1355.17,1723.81 1359.53,1724.67 1363.89,1725.9 1368.25,1727.11 1372.61,1728.5 1376.97,1729.91 1381.33,1731.35 
  1385.69,1732.81 1390.06,1734.43 1394.42,1736.09 1398.78,1737.5 1403.14,1738.57 1407.5,1739.36 1411.86,1740.34 1416.22,1741.5 1420.58,1742.89 1424.94,1744.38 
  1429.3,1745.88 1433.66,1747.37 1438.02,1748.94 1442.39,1750.56 1446.75,1752.24 1451.11,1753.86 1455.47,1755.18 1459.83,1756.18 1464.19,1757.27 1468.55,1758.52 
  1472.91,1759.99 1477.27,1761.52 1481.63,1762.9 1485.99,1764.34 1490.35,1765.68 1494.72,1767.23 1499.08,1768.76 1503.44,1770.27 1507.8,1771.48 1512.16,1772.32 
  1516.52,1773.21 1520.88,1774.29 1525.24,1775.71 1529.6,1777.16 1533.96,1778.6 1538.32,1780.21 1542.69,1781.75 1547.05,1783.35 1551.41,1785.03 1555.77,1786.68 
  1560.13,1788.11 1564.49,1789.16 1568.85,1789.95 1573.21,1791.01 1577.57,1792.4 1581.93,1793.8 1586.29,1795.31 1590.65,1796.71 1595.02,1797.79 1599.38,1799.07 
  1603.74,1800.44 1608.1,1801.83 1612.46,1803.07 1616.82,1803.99 1621.18,1804.74 1625.54,1805.44 1629.9,1806.42 1634.26,1807.6 1638.62,1808.75 1642.98,1809.85 
  1647.35,1810.94 1651.71,1811.98 1656.07,1813.45 1660.43,1814.87 1664.79,1816.16 1669.15,1817.1 1673.51,1817.8 1677.87,1818.65 1682.23,1819.69 1686.59,1820.93 
  1690.95,1822.14 1695.31,1823.15 1699.68,1824.14 1704.04,1825.38 1708.4,1826.64 1712.76,1827.86 1717.12,1829 1721.48,1829.63 1725.84,1829.86 1730.2,1830.16 
  1734.56,1830.68 1738.92,1831.48 1743.28,1832.39 1747.65,1833.12 1752.01,1833.68 1756.37,1834.41 1760.73,1835.08 1765.09,1835.93 1769.45,1836.66 1773.81,1837.23 
  1778.17,1837.44 1782.53,1837.68 1786.89,1838.09 1791.25,1838.64 1795.61,1839.45 1799.98,1840.19 1804.34,1840.79 1808.7,1841.36 1813.06,1841.92 1817.42,1842.44 
  1821.78,1843.12 1826.14,1843.57 1830.5,1843.71 1834.86,1843.71 1839.22,1843.96 1843.58,1844.31 1847.94,1845.01 1852.31,1845.56 1856.67,1845.96 1861.03,1846.19 
  1865.39,1846.81 1869.75,1847.37 1874.11,1847.9 1878.47,1848.23 1882.83,1848.24 1887.19,1848.12 1891.55,1848.29 1895.91,1848.56 1900.28,1849.09 1904.64,1849.58 
  1909,1850.09 1913.36,1850.48 1917.72,1851.05 1922.08,1851.54 1926.44,1851.93 1930.8,1852.34 1935.16,1852.31 1939.52,1852.26 1943.88,1852.23 1948.24,1852.37 
  1952.61,1852.68 1956.97,1853.03 1961.33,1853.48 1965.69,1853.82 1970.05,1854.14 1974.41,1854.54 1978.77,1854.95 1983.13,1855.23 1987.49,1855.11 1991.85,1854.8 
  1996.21,1854.7 2000.57,1854.8 2004.94,1855.1 2009.3,1855.33 2013.66,1855.45 2018.02,1855.74 2022.38,1855.99 2026.74,1856.29 2031.1,1856.77 2035.46,1857.09 
  2039.82,1857.11 2044.18,1856.86 2048.54,1856.77 2052.9,1856.78 2057.27,1857 2061.63,1857.35 2065.99,1857.72 2070.35,1858.03 2074.71,1858.41 2079.07,1859.02 
  2083.43,1859.41 2087.79,1859.93 2092.15,1860.04 2096.51,1859.86 2100.87,1859.76 2105.24,1859.73 2109.6,1860.01 2113.96,1860.44 2118.32,1860.8 2122.68,1861.05 
  2127.04,1861.42 2131.4,1861.98 2135.76,1862.62 2140.12,1863.29 2144.48,1863.73 2148.84,1863.95 2153.2,1864.14 2157.57,1864.46 2161.93,1865.1 2166.29,1865.83 
  2170.65,1866.46 2175.01,1867.08 2179.37,1867.9 2183.73,1868.52 2188.09,1869.24 2192.45,1870.02 2196.81,1870.66 2201.17,1870.95 2205.53,1871.02 2209.9,1871.08 
  2214.26,1871.52 2218.62,1872.05 2222.98,1872.74 2227.34,1873.24 2231.7,1873.81 2236.06,1874.37 2240.42,1874.98 2244.78,1875.66 2249.14,1876.32 2253.5,1876.64 
  2257.87,1876.8 2262.23,1877.01 2266.59,1877.53 2270.95,1878.2 2275.31,1878.93 2279.67,1879.55 2284.03,1880.03 2288.39,1880.46 2292.75,1881.17 2297.11,1881.93 
  2301.47,1882.59 2305.83,1883.03 2310.2,1883.11 2314.56,1883.3 2318.92,1883.67 2323.28,1884.2 2327.64,1884.72 2332,1885.32 2336.36,1885.8 2340.72,1886.47 
  2345.08,1887.18 2349.44,1887.94 2353.8,1888.61 2358.16,1888.96 2362.53,1888.84 2366.89,1888.74 2371.25,1888.88 2375.61,1889.34 2379.97,1889.95 2384.33,1890.41 
  2388.69,1890.7 2393.05,1891.06 2397.41,1891.65 2401.77,1892.37 2406.13,1893.06 2410.49,1893.54 2414.86,1893.59 2419.22,1893.63 2423.58,1893.86 2427.94,1894.5 
  2432.3,1895.11 2436.66,1895.69 2441.02,1896.36 2445.38,1896.94 2449.74,1897.43 2454.1,1898.24 2458.46,1898.9 2462.83,1899.39 2467.19,1899.51 2471.55,1899.64 
  2475.91,1899.86 2480.27,1900.39 2484.63,1900.99 2488.99,1901.79 2493.35,1902.34 2497.71,1903.07 2502.07,1903.8 2506.43,1904.77 2510.79,1905.69 2515.16,1906.35 
  2519.52,1906.62 2523.88,1906.76 2528.24,1907.03 2532.6,1907.58 2536.96,1908.31 2541.32,1909.04 2545.68,1909.72 2550.04,1910.34 2554.4,1911.05 2558.76,1911.88 
  2563.12,1912.74 2567.49,1913.43 2571.85,1913.81 2576.21,1914.06 2580.57,1914.3 2584.93,1914.77 2589.29,1915.44 2593.65,1916.14 2598.01,1916.85 2602.37,1917.52 
  2606.73,1918.27 2611.09,1919.15 2615.46,1920 2619.82,1920.8 2624.18,1921.23 2628.54,1921.69 2632.9,1922.15 2637.26,1922.91 2641.62,1923.86 2645.98,1924.86 
  2650.34,1925.84 2654.7,1926.94 2659.06,1927.87 2663.42,1929.02 2667.79,1930.17 2672.15,1931.16 2676.51,1931.89 2680.87,1932.2 2685.23,1932.49 2689.59,1933.02 
  2693.95,1933.83 2698.31,1934.78 2702.67,1935.64 2707.03,1936.36 2711.39,1937.09 2715.75,1937.91 2720.12,1938.83 2724.48,1939.75 2728.84,1940.42 2733.2,1940.7 
  2737.56,1940.96 2741.92,1941.22 2746.28,1941.72 2750.64,1942.4 2755,1943.11 2759.36,1943.63 2763.72,1944.12 2768.09,1944.69 2772.45,1945.29 2776.81,1945.93 
  2781.17,1946.36 2785.53,1946.38 2789.89,1946.45 2794.25,1946.58 2798.61,1946.96 2802.97,1947.55 2807.33,1948.18 2811.69,1948.6 2816.05,1949.08 2820.42,1949.66 
  2824.78,1950.3 2829.14,1950.88 2833.5,1951.25 2837.86,1951.28 2842.22,1951.02 2846.58,1950.91 2850.94,1950.9 2855.3,1951.01 2859.66,1951.23 2864.02,1951.36 
  2868.38,1951.62 2872.75,1951.91 2877.11,1952.32 2881.47,1952.67 2885.83,1952.9 2890.19,1952.65 2894.55,1952.06 2898.91,1951.65 2903.27,1951.34 2907.63,1951.3 
  2911.99,1951.4 2916.35,1951.4 2920.71,1951.23 2925.08,1951.23 2929.44,1951.24 2933.8,1951.41 2938.16,1951.51 2942.52,1951.33 2946.88,1950.79 2951.24,1950.35 
  2955.6,1950.01 2959.96,1949.94 2964.32,1949.99 2968.68,1949.99 2973.05,1950 2977.41,1949.87 2981.77,1949.83 2986.13,1949.8 2990.49,1949.79 2994.85,1949.47 
  2999.21,1948.88 3003.57,1948.36 3007.93,1948.03 3012.29,1947.92 3016.65,1948.13 3021.01,1948.16 3025.38,1948.17 3029.74,1948.07 3034.1,1948.22 3038.46,1948.4 
  3042.82,1948.5 3047.18,1948.29 3051.54,1947.79 3055.9,1947.19 3060.26,1946.78 3064.62,1946.72 3068.98,1946.83 3073.34,1946.87 
  "/>
<polyline clip-path="url(#clip412)" style="stroke:#c68125; stroke-linecap:round; stroke-linejoin:round; stroke-width:8; stroke-opacity:1; fill:none" points="
  426.307,1587.42 430.667,1587.42 435.028,1587.42 439.389,1587.42 443.75,1587.42 448.111,1587.42 452.472,1587.42 456.833,1587.42 461.193,1587.42 465.554,1587.42 
  469.915,1587.42 474.276,1587.42 478.637,1587.42 482.998,1587.42 487.359,1587.42 491.719,1587.42 496.08,1587.42 500.441,1587.42 504.802,1587.42 509.163,1587.42 
  513.524,1587.42 517.884,1587.42 522.245,1587.42 526.606,1587.42 530.967,1587.42 535.328,1587.42 539.689,1587.42 544.05,1587.42 548.41,1587.42 552.771,1587.42 
  557.132,1587.42 561.493,1587.42 565.854,1587.42 570.215,1587.42 574.576,1587.42 578.936,1587.42 583.297,1587.42 587.658,1587.42 592.019,1587.42 596.38,1587.42 
  600.741,1587.42 605.102,1587.42 609.462,1587.42 613.823,1587.42 618.184,1587.42 622.545,1587.42 626.906,1587.42 631.267,1587.42 635.628,1587.42 639.988,1587.42 
  644.349,1587.42 648.71,1587.42 653.071,1587.42 657.432,1587.42 661.793,1587.42 666.154,1587.42 670.514,1587.42 674.875,1587.42 679.236,1587.42 683.597,1587.42 
  687.958,1587.42 692.319,1587.42 696.679,1587.42 701.04,1587.42 705.401,1587.42 709.762,1587.42 714.123,1587.42 718.484,1587.42 722.845,1587.42 727.205,1587.42 
  731.566,1587.42 735.927,1587.42 740.288,1587.42 744.649,1587.42 749.01,1587.42 753.371,1587.42 757.731,1587.42 762.092,1587.42 766.453,1587.42 770.814,1587.42 
  775.175,1587.42 779.536,1587.42 783.897,1587.42 788.257,1587.42 792.618,1587.42 796.979,1587.42 801.34,1587.42 805.701,1587.42 810.062,1587.42 814.423,1587.42 
  818.783,1587.42 823.144,1587.42 827.505,1587.42 831.866,1587.42 836.227,1587.42 840.588,1587.42 844.949,1587.42 849.309,1587.42 853.67,1587.42 858.031,1587.42 
  862.392,1587.42 866.753,1587.42 871.114,1587.42 875.474,1587.42 879.835,1587.42 884.196,1587.42 888.557,1587.42 892.918,1587.42 897.279,1587.42 901.64,1587.42 
  906,1587.42 910.361,1587.42 914.722,1587.42 919.083,1587.42 923.444,1587.42 927.805,1587.42 932.166,1587.42 936.526,1587.42 940.887,1587.42 945.248,1587.42 
  949.609,1587.42 953.97,1587.42 958.331,1587.42 962.692,1587.42 967.052,1587.42 971.413,1587.42 975.774,1587.42 980.135,1587.42 984.496,1587.42 988.857,1587.42 
  993.218,1587.42 997.578,1587.42 1001.94,1587.42 1006.3,1587.42 1010.66,1587.42 1015.02,1587.42 1019.38,1587.42 1023.74,1587.42 1028.1,1587.42 1032.47,1587.42 
  1036.83,1587.42 1041.19,1587.42 1045.55,1587.42 1049.91,1587.42 1054.27,1587.42 1058.63,1587.42 1062.99,1587.42 1067.35,1587.42 1071.71,1587.42 1076.07,1587.42 
  1080.43,1587.42 1084.8,1587.42 1089.16,1587.42 1093.52,1587.42 1097.88,1587.42 1102.24,1587.42 1106.6,1587.42 1110.96,1587.42 1115.32,1587.42 1119.68,1587.42 
  1124.04,1587.42 1128.4,1587.42 1132.76,1587.42 1137.13,1587.42 1141.49,1587.42 1145.85,1587.42 1150.21,1587.42 1154.57,1587.42 1158.93,1587.42 1163.29,1587.42 
  1167.65,1587.42 1172.01,1587.42 1176.37,1587.42 1180.73,1587.42 1185.1,1587.42 1189.46,1587.42 1193.82,1587.42 1198.18,1587.42 1202.54,1587.42 1206.9,1587.42 
  1211.26,1587.42 1215.62,1587.42 1219.98,1587.42 1224.34,1587.42 1228.7,1587.42 1233.06,1587.42 1237.43,1587.42 1241.79,1587.42 1246.15,1587.42 1250.51,1587.42 
  1254.87,1587.42 1259.23,1587.42 1263.59,1587.42 1267.95,1587.42 1272.31,1587.42 1276.67,1587.42 1281.03,1587.42 1285.39,1587.42 1289.76,1587.42 1294.12,1587.42 
  1298.48,1587.42 1302.84,1587.42 1307.2,1587.42 1311.56,1587.42 1315.92,1587.42 1320.28,1587.42 1324.64,1587.42 1329,1587.42 1333.36,1587.42 1337.72,1587.42 
  1342.09,1587.42 1346.45,1587.42 1350.81,1587.42 1355.17,1587.42 1359.53,1587.42 1363.89,1587.42 1368.25,1587.42 1372.61,1587.42 1376.97,1587.42 1381.33,1587.42 
  1385.69,1587.42 1390.06,1587.42 1394.42,1587.42 1398.78,1587.42 1403.14,1587.42 1407.5,1587.42 1411.86,1587.42 1416.22,1587.42 1420.58,1587.42 1424.94,1587.42 
  1429.3,1587.42 1433.66,1587.42 1438.02,1587.42 1442.39,1587.42 1446.75,1587.42 1451.11,1587.42 1455.47,1587.42 1459.83,1587.42 1464.19,1587.42 1468.55,1587.42 
  1472.91,1587.42 1477.27,1587.42 1481.63,1587.42 1485.99,1587.42 1490.35,1587.42 1494.72,1587.42 1499.08,1587.42 1503.44,1587.42 1507.8,1587.42 1512.16,1587.42 
  1516.52,1587.42 1520.88,1587.42 1525.24,1587.42 1529.6,1587.42 1533.96,1587.42 1538.32,1587.42 1542.69,1587.42 1547.05,1587.42 1551.41,1587.42 1555.77,1587.42 
  1560.13,1587.42 1564.49,1587.42 1568.85,1587.42 1573.21,1587.42 1577.57,1587.42 1581.93,1587.42 1586.29,1587.42 1590.65,1587.42 1595.02,1587.42 1599.38,1587.42 
  1603.74,1587.42 1608.1,1587.42 1612.46,1587.42 1616.82,1587.42 1621.18,1587.42 1625.54,1587.42 1629.9,1587.42 1634.26,1587.42 1638.62,1587.42 1642.98,1587.42 
  1647.35,1587.42 1651.71,1587.42 1656.07,1587.42 1660.43,1587.42 1664.79,1587.42 1669.15,1587.42 1673.51,1587.42 1677.87,1587.42 1682.23,1587.42 1686.59,1587.42 
  1690.95,1587.42 1695.31,1587.42 1699.68,1587.42 1704.04,1587.42 1708.4,1587.42 1712.76,1587.42 1717.12,1587.42 1721.48,1587.42 1725.84,1587.42 1730.2,1587.42 
  1734.56,1587.42 1738.92,1587.42 1743.28,1587.42 1747.65,1587.42 1752.01,1587.42 1756.37,1587.42 1760.73,1587.42 1765.09,1587.42 1769.45,1587.42 1773.81,1587.42 
  1778.17,1587.42 1782.53,1587.42 1786.89,1587.42 1791.25,1587.42 1795.61,1587.42 1799.98,1587.42 1804.34,1587.42 1808.7,1587.42 1813.06,1587.42 1817.42,1587.42 
  1821.78,1587.42 1826.14,1587.42 1830.5,1587.42 1834.86,1587.42 1839.22,1587.42 1843.58,1587.42 1847.94,1587.42 1852.31,1587.42 1856.67,1587.42 1861.03,1587.42 
  1865.39,1587.42 1869.75,1587.42 1874.11,1587.42 1878.47,1587.42 1882.83,1587.42 1887.19,1587.42 1891.55,1587.42 1895.91,1587.42 1900.28,1587.42 1904.64,1587.42 
  1909,1587.42 1913.36,1587.42 1917.72,1587.42 1922.08,1587.42 1926.44,1587.42 1930.8,1587.42 1935.16,1587.42 1939.52,1587.42 1943.88,1587.42 1948.24,1587.42 
  1952.61,1587.42 1956.97,1587.42 1961.33,1587.42 1965.69,1587.42 1970.05,1587.42 1974.41,1587.42 1978.77,1587.42 1983.13,1587.42 1987.49,1587.42 1991.85,1587.42 
  1996.21,1587.42 2000.57,1587.42 2004.94,1587.42 2009.3,1587.42 2013.66,1587.42 2018.02,1587.42 2022.38,1587.42 2026.74,1587.42 2031.1,1587.42 2035.46,1587.42 
  2039.82,1587.42 2044.18,1587.42 2048.54,1587.42 2052.9,1587.42 2057.27,1587.42 2061.63,1587.42 2065.99,1587.42 2070.35,1587.42 2074.71,1587.42 2079.07,1587.42 
  2083.43,1587.42 2087.79,1587.42 2092.15,1587.42 2096.51,1587.42 2100.87,1587.42 2105.24,1587.42 2109.6,1587.42 2113.96,1587.42 2118.32,1587.42 2122.68,1587.42 
  2127.04,1587.42 2131.4,1587.42 2135.76,1587.42 2140.12,1587.42 2144.48,1587.42 2148.84,1587.42 2153.2,1587.42 2157.57,1587.42 2161.93,1587.42 2166.29,1587.42 
  2170.65,1587.42 2175.01,1587.42 2179.37,1587.42 2183.73,1587.42 2188.09,1587.42 2192.45,1587.42 2196.81,1587.42 2201.17,1587.42 2205.53,1587.42 2209.9,1587.42 
  2214.26,1587.42 2218.62,1587.42 2222.98,1587.42 2227.34,1587.42 2231.7,1587.42 2236.06,1587.42 2240.42,1587.42 2244.78,1587.42 2249.14,1587.42 2253.5,1587.42 
  2257.87,1587.42 2262.23,1587.42 2266.59,1587.42 2270.95,1587.42 2275.31,1587.42 2279.67,1587.42 2284.03,1587.42 2288.39,1587.42 2292.75,1587.42 2297.11,1587.42 
  2301.47,1587.42 2305.83,1587.42 2310.2,1587.42 2314.56,1587.42 2318.92,1587.42 2323.28,1587.42 2327.64,1587.42 2332,1587.42 2336.36,1587.42 2340.72,1587.42 
  2345.08,1587.42 2349.44,1587.42 2353.8,1587.42 2358.16,1587.42 2362.53,1587.42 2366.89,1587.42 2371.25,1587.42 2375.61,1587.42 2379.97,1587.42 2384.33,1587.42 
  2388.69,1587.42 2393.05,1587.42 2397.41,1587.42 2401.77,1587.42 2406.13,1587.42 2410.49,1587.42 2414.86,1587.42 2419.22,1587.42 2423.58,1587.42 2427.94,1587.42 
  2432.3,1587.42 2436.66,1587.42 2441.02,1587.42 2445.38,1587.42 2449.74,1587.42 2454.1,1587.42 2458.46,1587.42 2462.83,1587.42 2467.19,1587.42 2471.55,1587.42 
  2475.91,1587.42 2480.27,1587.42 2484.63,1587.42 2488.99,1587.42 2493.35,1587.42 2497.71,1587.42 2502.07,1587.42 2506.43,1587.42 2510.79,1587.42 2515.16,1587.42 
  2519.52,1587.42 2523.88,1587.42 2528.24,1587.42 2532.6,1587.42 2536.96,1587.42 2541.32,1587.42 2545.68,1587.42 2550.04,1587.42 2554.4,1587.42 2558.76,1587.42 
  2563.12,1587.42 2567.49,1587.42 2571.85,1587.42 2576.21,1587.42 2580.57,1587.42 2584.93,1587.42 2589.29,1587.42 2593.65,1587.42 2598.01,1587.42 2602.37,1587.42 
  2606.73,1587.42 2611.09,1587.42 2615.46,1587.42 2619.82,1587.42 2624.18,1587.42 2628.54,1587.42 2632.9,1587.42 2637.26,1587.42 2641.62,1587.42 2645.98,1587.42 
  2650.34,1587.42 2654.7,1587.42 2659.06,1587.42 2663.42,1587.42 2667.79,1587.42 2672.15,1587.42 2676.51,1587.42 2680.87,1587.42 2685.23,1587.42 2689.59,1587.42 
  2693.95,1587.42 2698.31,1587.42 2702.67,1587.42 2707.03,1587.42 2711.39,1587.42 2715.75,1587.42 2720.12,1587.42 2724.48,1587.42 2728.84,1587.42 2733.2,1587.42 
  2737.56,1587.42 2741.92,1587.42 2746.28,1587.42 2750.64,1587.42 2755,1587.42 2759.36,1587.42 2763.72,1587.42 2768.09,1587.42 2772.45,1587.42 2776.81,1587.42 
  2781.17,1587.42 2785.53,1587.42 2789.89,1587.42 2794.25,1587.42 2798.61,1587.42 2802.97,1587.42 2807.33,1587.42 2811.69,1587.42 2816.05,1587.42 2820.42,1587.42 
  2824.78,1587.42 2829.14,1587.42 2833.5,1587.42 2837.86,1587.42 2842.22,1587.42 2846.58,1587.42 2850.94,1587.42 2855.3,1587.42 2859.66,1587.42 2864.02,1587.42 
  2868.38,1587.42 2872.75,1587.42 2877.11,1587.42 2881.47,1587.42 2885.83,1587.42 2890.19,1587.42 2894.55,1587.42 2898.91,1587.42 2903.27,1587.42 2907.63,1587.42 
  2911.99,1587.42 2916.35,1587.42 2920.71,1587.42 2925.08,1587.42 2929.44,1587.42 2933.8,1587.42 2938.16,1587.42 2942.52,1587.42 2946.88,1587.42 2951.24,1587.42 
  2955.6,1587.42 2959.96,1587.42 2964.32,1587.42 2968.68,1587.42 2973.05,1587.42 2977.41,1587.42 2981.77,1587.42 2986.13,1587.42 2990.49,1587.42 2994.85,1587.42 
  2999.21,1587.42 3003.57,1587.42 3007.93,1587.42 3012.29,1587.42 3016.65,1587.42 3021.01,1587.42 3025.38,1587.42 3029.74,1587.42 3034.1,1587.42 3038.46,1587.42 
  3042.82,1587.42 3047.18,1587.42 3051.54,1587.42 3055.9,1587.42 3060.26,1587.42 3064.62,1587.42 3068.98,1587.42 3073.34,1587.42 
  "/>
<path clip-path="url(#clip410)" d="
M440.424 579.176 L815.778 579.176 L815.778 112.616 L440.424 112.616  Z
  " fill="#ffffff" fill-rule="evenodd" fill-opacity="1"/>
<polyline clip-path="url(#clip410)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="
  440.424,579.176 815.778,579.176 815.778,112.616 440.424,112.616 440.424,579.176 
  "/>
<polyline clip-path="url(#clip410)" style="stroke:#009af9; stroke-linecap:round; stroke-linejoin:round; stroke-width:8; stroke-opacity:1; fill:none" points="
  471.6,164.456 658.658,164.456 
  "/>
<path clip-path="url(#clip410)" d="M689.834 147.176 L719.07 147.176 L719.07 151.111 L706.801 151.111 L706.801 181.736 L702.102 181.736 L702.102 151.111 L689.834 151.111 L689.834 147.176 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M731.454 151.782 L725.112 168.981 L737.82 168.981 L731.454 151.782 M728.815 147.176 L734.116 147.176 L747.287 181.736 L742.426 181.736 L739.278 172.87 L723.699 172.87 L720.551 181.736 L715.621 181.736 L728.815 147.176 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M752.334 147.176 L757.009 147.176 L757.009 181.736 L752.334 181.736 L752.334 147.176 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><polyline clip-path="url(#clip410)" style="stroke:#e26f46; stroke-linecap:round; stroke-linejoin:round; stroke-width:8; stroke-opacity:1; fill:none" points="
  471.6,216.296 658.658,216.296 
  "/>
<path clip-path="url(#clip410)" d="M689.834 199.016 L719.07 199.016 L719.07 202.951 L706.801 202.951 L706.801 233.576 L702.102 233.576 L702.102 202.951 L689.834 202.951 L689.834 199.016 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M717.959 199.016 L747.195 199.016 L747.195 202.951 L734.926 202.951 L734.926 233.576 L730.227 233.576 L730.227 202.951 L717.959 202.951 L717.959 199.016 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><polyline clip-path="url(#clip410)" style="stroke:#3da44d; stroke-linecap:round; stroke-linejoin:round; stroke-width:8; stroke-opacity:1; fill:none" points="
  471.6,268.136 658.658,268.136 
  "/>
<path clip-path="url(#clip410)" d="M689.834 250.856 L719.07 250.856 L719.07 254.791 L706.801 254.791 L706.801 285.416 L702.102 285.416 L702.102 254.791 L689.834 254.791 L689.834 250.856 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M728.26 254.699 L728.26 281.573 L733.908 281.573 Q741.06 281.573 744.371 278.333 Q747.704 275.092 747.704 268.101 Q747.704 261.157 744.371 257.939 Q741.06 254.699 733.908 254.699 L728.26 254.699 M723.584 250.856 L733.19 250.856 Q743.236 250.856 747.935 255.046 Q752.635 259.212 752.635 268.101 Q752.635 277.036 747.912 281.226 Q743.19 285.416 733.19 285.416 L723.584 285.416 L723.584 250.856 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M764.764 268.911 L764.764 281.573 L772.264 281.573 Q776.037 281.573 777.843 280.023 Q779.671 278.448 779.671 275.231 Q779.671 271.99 777.843 270.462 Q776.037 268.911 772.264 268.911 L764.764 268.911 M764.764 254.699 L764.764 265.115 L771.685 265.115 Q775.111 265.115 776.778 263.842 Q778.468 262.546 778.468 259.907 Q778.468 257.291 776.778 255.995 Q775.111 254.699 771.685 254.699 L764.764 254.699 M760.088 250.856 L772.033 250.856 Q777.38 250.856 780.273 253.078 Q783.167 255.3 783.167 259.398 Q783.167 262.569 781.685 264.444 Q780.204 266.319 777.333 266.782 Q780.783 267.523 782.681 269.884 Q784.602 272.222 784.602 275.74 Q784.602 280.37 781.454 282.893 Q778.306 285.416 772.496 285.416 L760.088 285.416 L760.088 250.856 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><polyline clip-path="url(#clip410)" style="stroke:#c271d2; stroke-linecap:round; stroke-linejoin:round; stroke-width:8; stroke-opacity:1; fill:none" points="
  471.6,319.976 658.658,319.976 
  "/>
<path clip-path="url(#clip410)" d="M689.834 302.696 L719.07 302.696 L719.07 306.631 L706.801 306.631 L706.801 337.256 L702.102 337.256 L702.102 306.631 L689.834 306.631 L689.834 302.696 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M746.685 305.358 L746.685 310.289 Q744.324 308.09 741.639 307.002 Q738.977 305.914 735.968 305.914 Q730.042 305.914 726.894 309.548 Q723.746 313.159 723.746 320.011 Q723.746 326.839 726.894 330.474 Q730.042 334.085 735.968 334.085 Q738.977 334.085 741.639 332.997 Q744.324 331.909 746.685 329.71 L746.685 334.594 Q744.232 336.261 741.477 337.094 Q738.746 337.927 735.69 337.927 Q727.843 337.927 723.329 333.136 Q718.815 328.321 718.815 320.011 Q718.815 311.677 723.329 306.886 Q727.843 302.071 735.69 302.071 Q738.792 302.071 741.523 302.904 Q744.278 303.715 746.685 305.358 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M758.584 320.751 L758.584 333.413 L766.084 333.413 Q769.857 333.413 771.662 331.863 Q773.491 330.288 773.491 327.071 Q773.491 323.83 771.662 322.302 Q769.857 320.751 766.084 320.751 L758.584 320.751 M758.584 306.539 L758.584 316.955 L765.505 316.955 Q768.931 316.955 770.597 315.682 Q772.287 314.386 772.287 311.747 Q772.287 309.131 770.597 307.835 Q768.931 306.539 765.505 306.539 L758.584 306.539 M753.908 302.696 L765.852 302.696 Q771.199 302.696 774.093 304.918 Q776.986 307.14 776.986 311.238 Q776.986 314.409 775.505 316.284 Q774.023 318.159 771.153 318.622 Q774.602 319.363 776.5 321.724 Q778.421 324.062 778.421 327.58 Q778.421 332.21 775.273 334.733 Q772.125 337.256 766.315 337.256 L753.908 337.256 L753.908 302.696 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><polyline clip-path="url(#clip410)" style="stroke:#ac8d18; stroke-linecap:round; stroke-linejoin:round; stroke-width:8; stroke-opacity:1; fill:none" points="
  471.6,371.816 658.658,371.816 
  "/>
<path clip-path="url(#clip410)" d="M689.834 354.536 L719.07 354.536 L719.07 358.471 L706.801 358.471 L706.801 389.096 L702.102 389.096 L702.102 358.471 L689.834 358.471 L689.834 354.536 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M746.685 357.198 L746.685 362.129 Q744.324 359.93 741.639 358.842 Q738.977 357.754 735.968 357.754 Q730.042 357.754 726.894 361.388 Q723.746 364.999 723.746 371.851 Q723.746 378.679 726.894 382.314 Q730.042 385.925 735.968 385.925 Q738.977 385.925 741.639 384.837 Q744.324 383.749 746.685 381.55 L746.685 386.434 Q744.232 388.101 741.477 388.934 Q738.746 389.767 735.69 389.767 Q727.843 389.767 723.329 384.976 Q718.815 380.161 718.815 371.851 Q718.815 363.517 723.329 358.726 Q727.843 353.911 735.69 353.911 Q738.792 353.911 741.523 354.744 Q744.278 355.555 746.685 357.198 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M777.472 384.166 L777.472 374.883 L769.833 374.883 L769.833 371.041 L782.102 371.041 L782.102 385.878 Q779.394 387.8 776.13 388.795 Q772.866 389.767 769.162 389.767 Q761.06 389.767 756.477 385.045 Q751.917 380.3 751.917 371.851 Q751.917 363.379 756.477 358.656 Q761.06 353.911 769.162 353.911 Q772.542 353.911 775.574 354.744 Q778.63 355.578 781.199 357.198 L781.199 362.175 Q778.607 359.976 775.69 358.865 Q772.773 357.754 769.556 357.754 Q763.213 357.754 760.019 361.295 Q756.847 364.837 756.847 371.851 Q756.847 378.841 760.019 382.383 Q763.213 385.925 769.556 385.925 Q772.033 385.925 773.977 385.508 Q775.921 385.068 777.472 384.166 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><polyline clip-path="url(#clip410)" style="stroke:#00a9ad; stroke-linecap:round; stroke-linejoin:round; stroke-width:8; stroke-opacity:1; fill:none" points="
  471.6,423.656 658.658,423.656 
  "/>
<path clip-path="url(#clip410)" d="M689.834 406.376 L694.533 406.376 L694.533 427.371 Q694.533 432.927 696.547 435.381 Q698.561 437.811 703.075 437.811 Q707.565 437.811 709.579 435.381 Q711.593 432.927 711.593 427.371 L711.593 406.376 L716.292 406.376 L716.292 427.95 Q716.292 434.709 712.936 438.158 Q709.602 441.607 703.075 441.607 Q696.524 441.607 693.167 438.158 Q689.834 434.709 689.834 427.95 L689.834 406.376 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M720.274 406.376 L749.51 406.376 L749.51 410.311 L737.241 410.311 L737.241 440.936 L732.542 440.936 L732.542 410.311 L720.274 410.311 L720.274 406.376 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M777.125 409.038 L777.125 413.969 Q774.764 411.77 772.079 410.682 Q769.417 409.594 766.408 409.594 Q760.482 409.594 757.334 413.228 Q754.185 416.839 754.185 423.691 Q754.185 430.519 757.334 434.154 Q760.482 437.765 766.408 437.765 Q769.417 437.765 772.079 436.677 Q774.764 435.589 777.125 433.39 L777.125 438.274 Q774.671 439.941 771.917 440.774 Q769.185 441.607 766.13 441.607 Q758.283 441.607 753.769 436.816 Q749.255 432.001 749.255 423.691 Q749.255 415.357 753.769 410.566 Q758.283 405.751 766.13 405.751 Q769.232 405.751 771.963 406.584 Q774.718 407.395 777.125 409.038 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><polyline clip-path="url(#clip410)" style="stroke:#ed5d92; stroke-linecap:round; stroke-linejoin:round; stroke-width:8; stroke-opacity:1; fill:none" points="
  471.6,475.496 658.658,475.496 
  "/>
<path clip-path="url(#clip410)" d="M689.834 458.216 L694.533 458.216 L694.533 479.211 Q694.533 484.767 696.547 487.221 Q698.561 489.651 703.075 489.651 Q707.565 489.651 709.579 487.221 Q711.593 484.767 711.593 479.211 L711.593 458.216 L716.292 458.216 L716.292 479.79 Q716.292 486.549 712.936 489.998 Q709.602 493.447 703.075 493.447 Q696.524 493.447 693.167 489.998 Q689.834 486.549 689.834 479.79 L689.834 458.216 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M720.274 458.216 L749.51 458.216 L749.51 462.151 L737.241 462.151 L737.241 492.776 L732.542 492.776 L732.542 462.151 L720.274 462.151 L720.274 458.216 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M755.25 488.841 L762.889 488.841 L762.889 462.475 L754.579 464.142 L754.579 459.883 L762.843 458.216 L767.519 458.216 L767.519 488.841 L775.158 488.841 L775.158 492.776 L755.25 492.776 L755.25 488.841 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><polyline clip-path="url(#clip410)" style="stroke:#c68125; stroke-linecap:round; stroke-linejoin:round; stroke-width:8; stroke-opacity:1; fill:none" points="
  471.6,527.336 658.658,527.336 
  "/>
<path clip-path="url(#clip410)" d="M715.389 539.686 L715.389 530.403 L707.75 530.403 L707.75 526.561 L720.019 526.561 L720.019 541.398 Q717.311 543.32 714.047 544.315 Q710.783 545.287 707.079 545.287 Q698.977 545.287 694.394 540.565 Q689.834 535.82 689.834 527.371 Q689.834 518.899 694.394 514.176 Q698.977 509.431 707.079 509.431 Q710.459 509.431 713.491 510.264 Q716.547 511.098 719.116 512.718 L719.116 517.695 Q716.524 515.496 713.607 514.385 Q710.69 513.274 707.473 513.274 Q701.13 513.274 697.936 516.815 Q694.764 520.357 694.764 527.371 Q694.764 534.361 697.936 537.903 Q701.13 541.445 707.473 541.445 Q709.95 541.445 711.894 541.028 Q713.838 540.588 715.389 539.686 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M733.236 513.899 L733.236 526.885 L739.116 526.885 Q742.38 526.885 744.162 525.195 Q745.945 523.505 745.945 520.38 Q745.945 517.278 744.162 515.588 Q742.38 513.899 739.116 513.899 L733.236 513.899 M728.561 510.056 L739.116 510.056 Q744.926 510.056 747.889 512.695 Q750.875 515.311 750.875 520.38 Q750.875 525.496 747.889 528.111 Q744.926 530.727 739.116 530.727 L733.236 530.727 L733.236 544.616 L728.561 544.616 L728.561 510.056 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /><path clip-path="url(#clip410)" d="M777.866 511.19 L777.866 515.75 Q775.204 514.477 772.843 513.852 Q770.482 513.227 768.283 513.227 Q764.463 513.227 762.38 514.709 Q760.32 516.19 760.32 518.922 Q760.32 521.213 761.685 522.394 Q763.074 523.551 766.917 524.269 L769.741 524.848 Q774.972 525.843 777.449 528.366 Q779.949 530.866 779.949 535.079 Q779.949 540.102 776.57 542.695 Q773.213 545.287 766.709 545.287 Q764.255 545.287 761.477 544.732 Q758.722 544.176 755.759 543.088 L755.759 538.273 Q758.607 539.871 761.338 540.681 Q764.07 541.491 766.709 541.491 Q770.713 541.491 772.889 539.917 Q775.065 538.343 775.065 535.426 Q775.065 532.88 773.491 531.445 Q771.94 530.01 768.375 529.292 L765.528 528.736 Q760.297 527.695 757.959 525.473 Q755.621 523.25 755.621 519.292 Q755.621 514.709 758.838 512.07 Q762.079 509.431 767.75 509.431 Q770.181 509.431 772.704 509.871 Q775.227 510.311 777.866 511.19 Z" fill="#000000" fill-rule="evenodd" fill-opacity="1" /></svg>


